# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v20.enums.types import batch_job_status


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.resources",
    marshal="google.ads.googleads.v20",
    manifest={
        "BatchJob",
    },
)


class BatchJob(proto.Message):
    r"""A list of mutates being processed asynchronously. The mutates
    are uploaded by the user. The mutates themselves aren't readable
    and the results of the job can only be read using
    BatchJobService.ListBatchJobResults.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Immutable. The resource name of the batch job. Batch job
            resource names have the form:

            ``customers/{customer_id}/batchJobs/{batch_job_id}``
        id (int):
            Output only. ID of this batch job.

            This field is a member of `oneof`_ ``_id``.
        next_add_sequence_token (str):
            Output only. The next sequence token to use
            when adding operations. Only set when the batch
            job status is PENDING.

            This field is a member of `oneof`_ ``_next_add_sequence_token``.
        metadata (google.ads.googleads.v20.resources.types.BatchJob.BatchJobMetadata):
            Output only. Contains additional information
            about this batch job.
        status (google.ads.googleads.v20.enums.types.BatchJobStatusEnum.BatchJobStatus):
            Output only. Status of this batch job.
        long_running_operation (str):
            Output only. The resource name of the
            long-running operation that can be used to poll
            for completion. Only set when the batch job
            status is RUNNING or DONE.

            This field is a member of `oneof`_ ``_long_running_operation``.
    """

    class BatchJobMetadata(proto.Message):
        r"""Additional information about the batch job. This message is
        also used as metadata returned in batch job Long Running
        Operations.


        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            creation_date_time (str):
                Output only. The time when this batch job was
                created. Formatted as yyyy-mm-dd hh:mm:ss.
                Example: "2018-03-05 09:15:00".

                This field is a member of `oneof`_ ``_creation_date_time``.
            start_date_time (str):
                Output only. The time when this batch job
                started running. Formatted as yyyy-mm-dd
                hh:mm:ss. Example: "2018-03-05 09:15:30".

                This field is a member of `oneof`_ ``_start_date_time``.
            completion_date_time (str):
                Output only. The time when this batch job was
                completed. Formatted as yyyy-MM-dd HH:mm:ss.
                Example: "2018-03-05 09:16:00".

                This field is a member of `oneof`_ ``_completion_date_time``.
            estimated_completion_ratio (float):
                Output only. The fraction (between 0.0 and
                1.0) of mutates that have been processed. This
                is empty if the job hasn't started running yet.

                This field is a member of `oneof`_ ``_estimated_completion_ratio``.
            operation_count (int):
                Output only. The number of mutate operations
                in the batch job.

                This field is a member of `oneof`_ ``_operation_count``.
            executed_operation_count (int):
                Output only. The number of mutate operations
                executed by the batch job. Present only if the
                job has started running.

                This field is a member of `oneof`_ ``_executed_operation_count``.
            execution_limit_seconds (int):
                Immutable. The approximate upper bound for
                how long a batch job can be executed, in
                seconds. If the job runs more than the given
                upper bound, the job will be canceled.

                This field is a member of `oneof`_ ``_execution_limit_seconds``.
        """

        creation_date_time: str = proto.Field(
            proto.STRING,
            number=8,
            optional=True,
        )
        start_date_time: str = proto.Field(
            proto.STRING,
            number=7,
            optional=True,
        )
        completion_date_time: str = proto.Field(
            proto.STRING,
            number=9,
            optional=True,
        )
        estimated_completion_ratio: float = proto.Field(
            proto.DOUBLE,
            number=10,
            optional=True,
        )
        operation_count: int = proto.Field(
            proto.INT64,
            number=11,
            optional=True,
        )
        executed_operation_count: int = proto.Field(
            proto.INT64,
            number=12,
            optional=True,
        )
        execution_limit_seconds: int = proto.Field(
            proto.INT32,
            number=13,
            optional=True,
        )

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    id: int = proto.Field(
        proto.INT64,
        number=7,
        optional=True,
    )
    next_add_sequence_token: str = proto.Field(
        proto.STRING,
        number=8,
        optional=True,
    )
    metadata: BatchJobMetadata = proto.Field(
        proto.MESSAGE,
        number=4,
        message=BatchJobMetadata,
    )
    status: batch_job_status.BatchJobStatusEnum.BatchJobStatus = proto.Field(
        proto.ENUM,
        number=5,
        enum=batch_job_status.BatchJobStatusEnum.BatchJobStatus,
    )
    long_running_operation: str = proto.Field(
        proto.STRING,
        number=9,
        optional=True,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.enums",
    marshal="google.ads.googleads.v20",
    manifest={
        "LocalServicesLeadSurveySatisfiedReasonEnum",
    },
)


class LocalServicesLeadSurveySatisfiedReasonEnum(proto.Message):
    r"""Container for enum describing possible survey satisfied
    reasons for a lead.

    """

    class SurveySatisfiedReason(proto.Enum):
        r"""Provider's reason for being satisfied with the lead.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            OTHER_SATISFIED_REASON (2):
                Other reasons.
            BOOKED_CUSTOMER (3):
                Lead converted into a booked customer or
                client.
            LIKELY_BOOKED_CUSTOMER (4):
                Lead could convert into a booked customer or
                client soon.
            SERVICE_RELATED (5):
                Lead was related to the services the business
                offers.
            HIGH_VALUE_SERVICE (6):
                Lead was for a service that generates high
                value for the business.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        OTHER_SATISFIED_REASON = 2
        BOOKED_CUSTOMER = 3
        LIKELY_BOOKED_CUSTOMER = 4
        SERVICE_RELATED = 5
        HIGH_VALUE_SERVICE = 6


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v22.common.types import bidding
from google.ads.googleads.v22.enums.types import bidding_strategy_status
from google.ads.googleads.v22.enums.types import bidding_strategy_type


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.resources",
    marshal="google.ads.googleads.v22",
    manifest={
        "BiddingStrategy",
    },
)


class BiddingStrategy(proto.Message):
    r"""A bidding strategy.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Immutable. The resource name of the bidding strategy.
            Bidding strategy resource names have the form:

            ``customers/{customer_id}/biddingStrategies/{bidding_strategy_id}``
        id (int):
            Output only. The ID of the bidding strategy.

            This field is a member of `oneof`_ ``_id``.
        name (str):
            The name of the bidding strategy.
            All bidding strategies within an account must be
            named distinctly.

            The length of this string should be between 1
            and 255, inclusive, in UTF-8 bytes, (trimmed).

            This field is a member of `oneof`_ ``_name``.
        status (google.ads.googleads.v22.enums.types.BiddingStrategyStatusEnum.BiddingStrategyStatus):
            Output only. The status of the bidding
            strategy.
            This field is read-only.
        type_ (google.ads.googleads.v22.enums.types.BiddingStrategyTypeEnum.BiddingStrategyType):
            Output only. The type of the bidding
            strategy. Create a bidding strategy by setting
            the bidding scheme.

            This field is read-only.
        currency_code (str):
            Immutable. The currency used by the bidding strategy (ISO
            4217 three-letter code).

            For bidding strategies in manager customers, this currency
            can be set on creation and defaults to the manager
            customer's currency. For serving customers, this field
            cannot be set; all strategies in a serving customer
            implicitly use the serving customer's currency. In all cases
            the effective_currency_code field returns the currency used
            by the strategy.
        effective_currency_code (str):
            Output only. The currency used by the bidding strategy (ISO
            4217 three-letter code).

            For bidding strategies in manager customers, this is the
            currency set by the advertiser when creating the strategy.
            For serving customers, this is the customer's currency_code.

            Bidding strategy metrics are reported in this currency.

            This field is read-only.

            This field is a member of `oneof`_ ``_effective_currency_code``.
        aligned_campaign_budget_id (int):
            ID of the campaign budget that this portfolio
            bidding strategy is aligned with. When a
            portfolio and a campaign budget are aligned,
            that means that they are attached to the same
            set of campaigns. After a bidding strategy is
            aligned with a campaign budget, campaigns that
            are added to the bidding strategy must also use
            the aligned campaign budget.
        campaign_count (int):
            Output only. The number of campaigns attached
            to this bidding strategy.
            This field is read-only.

            This field is a member of `oneof`_ ``_campaign_count``.
        non_removed_campaign_count (int):
            Output only. The number of non-removed
            campaigns attached to this bidding strategy.

            This field is read-only.

            This field is a member of `oneof`_ ``_non_removed_campaign_count``.
        enhanced_cpc (google.ads.googleads.v22.common.types.EnhancedCpc):
            A bidding strategy that raises bids for
            clicks that seem more likely to lead to a
            conversion and lowers them for clicks where they
            seem less likely.

            This field is a member of `oneof`_ ``scheme``.
        maximize_conversion_value (google.ads.googleads.v22.common.types.MaximizeConversionValue):
            An automated bidding strategy to help get the
            most conversion value for your campaigns while
            spending your budget.

            This field is a member of `oneof`_ ``scheme``.
        maximize_conversions (google.ads.googleads.v22.common.types.MaximizeConversions):
            An automated bidding strategy to help get the
            most conversions for your campaigns while
            spending your budget.

            This field is a member of `oneof`_ ``scheme``.
        target_cpa (google.ads.googleads.v22.common.types.TargetCpa):
            A bidding strategy that sets bids to help get
            as many conversions as possible at the target
            cost-per-acquisition (CPA) you set.

            This field is a member of `oneof`_ ``scheme``.
        target_impression_share (google.ads.googleads.v22.common.types.TargetImpressionShare):
            A bidding strategy that automatically
            optimizes towards a chosen percentage of
            impressions.

            This field is a member of `oneof`_ ``scheme``.
        target_roas (google.ads.googleads.v22.common.types.TargetRoas):
            A bidding strategy that helps you maximize
            revenue while averaging a specific target Return
            On Ad Spend (ROAS).

            This field is a member of `oneof`_ ``scheme``.
        target_spend (google.ads.googleads.v22.common.types.TargetSpend):
            A bid strategy that sets your bids to help
            get as many clicks as possible within your
            budget.

            This field is a member of `oneof`_ ``scheme``.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    id: int = proto.Field(
        proto.INT64,
        number=16,
        optional=True,
    )
    name: str = proto.Field(
        proto.STRING,
        number=17,
        optional=True,
    )
    status: (
        bidding_strategy_status.BiddingStrategyStatusEnum.BiddingStrategyStatus
    ) = proto.Field(
        proto.ENUM,
        number=15,
        enum=bidding_strategy_status.BiddingStrategyStatusEnum.BiddingStrategyStatus,
    )
    type_: bidding_strategy_type.BiddingStrategyTypeEnum.BiddingStrategyType = (
        proto.Field(
            proto.ENUM,
            number=5,
            enum=bidding_strategy_type.BiddingStrategyTypeEnum.BiddingStrategyType,
        )
    )
    currency_code: str = proto.Field(
        proto.STRING,
        number=23,
    )
    effective_currency_code: str = proto.Field(
        proto.STRING,
        number=20,
        optional=True,
    )
    aligned_campaign_budget_id: int = proto.Field(
        proto.INT64,
        number=25,
    )
    campaign_count: int = proto.Field(
        proto.INT64,
        number=18,
        optional=True,
    )
    non_removed_campaign_count: int = proto.Field(
        proto.INT64,
        number=19,
        optional=True,
    )
    enhanced_cpc: bidding.EnhancedCpc = proto.Field(
        proto.MESSAGE,
        number=7,
        oneof="scheme",
        message=bidding.EnhancedCpc,
    )
    maximize_conversion_value: bidding.MaximizeConversionValue = proto.Field(
        proto.MESSAGE,
        number=21,
        oneof="scheme",
        message=bidding.MaximizeConversionValue,
    )
    maximize_conversions: bidding.MaximizeConversions = proto.Field(
        proto.MESSAGE,
        number=22,
        oneof="scheme",
        message=bidding.MaximizeConversions,
    )
    target_cpa: bidding.TargetCpa = proto.Field(
        proto.MESSAGE,
        number=9,
        oneof="scheme",
        message=bidding.TargetCpa,
    )
    target_impression_share: bidding.TargetImpressionShare = proto.Field(
        proto.MESSAGE,
        number=48,
        oneof="scheme",
        message=bidding.TargetImpressionShare,
    )
    target_roas: bidding.TargetRoas = proto.Field(
        proto.MESSAGE,
        number=11,
        oneof="scheme",
        message=bidding.TargetRoas,
    )
    target_spend: bidding.TargetSpend = proto.Field(
        proto.MESSAGE,
        number=12,
        oneof="scheme",
        message=bidding.TargetSpend,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



#!/usr/bin/env python
# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Adds a channel-level data exclusion for Smart Bidding.

The exclusion specifically excludes conversions from being used by Smart Bidding
for the time interval specified.

For more information on using data exclusions, see:
https://developers.google.com/google-ads/api/docs/campaigns/bidding/data-exclusions
"""


import argparse
import sys
from uuid import uuid4

from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.v22.resources.types.bidding_data_exclusion import (
    BiddingDataExclusion,
)
from google.ads.googleads.v22.services.services.bidding_data_exclusion_service import (
    BiddingDataExclusionServiceClient,
)
from google.ads.googleads.v22.services.types.bidding_data_exclusion_service import (
    BiddingDataExclusionOperation,
    MutateBiddingDataExclusionsResponse,
)


def main(
    client: GoogleAdsClient,
    customer_id: str,
    start_date_time: str,
    end_date_time: str,
) -> None:
    """The main method that creates all necessary entities for the example.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
        start_date_time: a str of the start date for the exclusion period.
        end_date_time: a str of the end date for the exclusion period.
    """
    # [START add_bidding_data_exclusion]
    bidding_data_exclusion_service: BiddingDataExclusionServiceClient = (
        client.get_service("BiddingDataExclusionService")
    )
    operation: BiddingDataExclusionOperation = client.get_type(
        "BiddingDataExclusionOperation"
    )
    bidding_data_exclusion: BiddingDataExclusion = operation.create
    # A unique name is required for every data exclusion
    bidding_data_exclusion.name = f"Data exclusion #{uuid4()}"
    # The CHANNEL scope applies the data exclusion to all campaigns of specific
    # advertising channel types. In this example, the exclusion will only
    # apply to Search campaigns. Use the CAMPAIGN scope to instead limit the
    # scope to specific campaigns.
    bidding_data_exclusion.scope = (
        client.enums.SeasonalityEventScopeEnum.CHANNEL
    )
    bidding_data_exclusion.advertising_channel_types.append(
        client.enums.AdvertisingChannelTypeEnum.SEARCH
    )
    # If setting scope CAMPAIGN, add individual campaign resource name(s)
    # according to the commented out line below.
    #
    # bidding_data_exclusion.campaigns.append(
    #     "INSERT_CAMPAIGN_RESOURCE_NAME_HERE"
    # )

    bidding_data_exclusion.start_date_time = start_date_time
    bidding_data_exclusion.end_date_time = end_date_time

    response: MutateBiddingDataExclusionsResponse = (
        bidding_data_exclusion_service.mutate_bidding_data_exclusions(
            customer_id=customer_id, operations=[operation]
        )
    )

    resource_name: str = response.results[0].resource_name

    print(f"Added data exclusion with resource name: '{resource_name}'")
    # [END add_bidding_data_exclusion]


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description="Adds a data exclusion for conversions in Smart Bidding "
        "for the given time interval."
    )
    # The following argument(s) should be provided to run the example.
    parser.add_argument(
        "-c",
        "--customer_id",
        type=str,
        required=True,
        help="The Google Ads customer ID.",
    )
    parser.add_argument(
        "-s",
        "--start_date_time",
        type=str,
        required=True,
        help="The start date for the exclusion period, must be in the format: "
        "'yyyy-MM-dd HH:mm:ss'.",
    )
    parser.add_argument(
        "-e",
        "--end_date_time",
        type=str,
        required=True,
        help="The end date for the exclusion period, must be in the format: "
        "'yyyy-MM-dd HH:mm:ss'.",
    )

    args: argparse.Namespace = parser.parse_args()

    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client: GoogleAdsClient = GoogleAdsClient.load_from_storage(
        version="v22"
    )

    try:
        main(
            googleads_client,
            args.customer_id,
            args.start_date_time,
            args.end_date_time,
        )
    except GoogleAdsException as ex:
        print(
            f'Request with ID "{ex.request_id}" failed with status '
            f'"{ex.error.code().name}" and includes the following errors:'
        )
        for error in ex.failure.errors:
            print(f'Error with message "{error.message}".')
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(f"\t\tOn field: {field_path_element.field_name}")
        sys.exit(1)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v20.enums.types import (
    response_content_type as gage_response_content_type,
)
from google.ads.googleads.v20.enums.types import (
    smart_campaign_not_eligible_reason,
)
from google.ads.googleads.v20.enums.types import (
    smart_campaign_status as gage_smart_campaign_status,
)
from google.ads.googleads.v20.resources.types import (
    smart_campaign_setting as gagr_smart_campaign_setting,
)
from google.protobuf import field_mask_pb2  # type: ignore
from google.rpc import status_pb2  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.services",
    marshal="google.ads.googleads.v20",
    manifest={
        "GetSmartCampaignStatusRequest",
        "SmartCampaignNotEligibleDetails",
        "SmartCampaignEligibleDetails",
        "SmartCampaignPausedDetails",
        "SmartCampaignRemovedDetails",
        "SmartCampaignEndedDetails",
        "GetSmartCampaignStatusResponse",
        "MutateSmartCampaignSettingsRequest",
        "SmartCampaignSettingOperation",
        "MutateSmartCampaignSettingsResponse",
        "MutateSmartCampaignSettingResult",
    },
)


class GetSmartCampaignStatusRequest(proto.Message):
    r"""Request message for
    [SmartCampaignSettingService.GetSmartCampaignStatus][google.ads.googleads.v20.services.SmartCampaignSettingService.GetSmartCampaignStatus].

    Attributes:
        resource_name (str):
            Required. The resource name of the Smart
            campaign setting belonging to the Smart campaign
            to fetch the status of.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class SmartCampaignNotEligibleDetails(proto.Message):
    r"""Details related to Smart campaigns that are not eligible to
    serve.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        not_eligible_reason (google.ads.googleads.v20.enums.types.SmartCampaignNotEligibleReasonEnum.SmartCampaignNotEligibleReason):
            The reason why the Smart campaign is not
            eligible to serve.

            This field is a member of `oneof`_ ``_not_eligible_reason``.
    """

    not_eligible_reason: (
        smart_campaign_not_eligible_reason.SmartCampaignNotEligibleReasonEnum.SmartCampaignNotEligibleReason
    ) = proto.Field(
        proto.ENUM,
        number=1,
        optional=True,
        enum=smart_campaign_not_eligible_reason.SmartCampaignNotEligibleReasonEnum.SmartCampaignNotEligibleReason,
    )


class SmartCampaignEligibleDetails(proto.Message):
    r"""Details related to Smart campaigns that are eligible to
    serve.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        last_impression_date_time (str):
            The timestamp of the last impression observed
            in the last 48 hours for this campaign.
            The timestamp is in the customer’s timezone and
            in “yyyy-MM-dd HH:mm:ss” format.

            This field is a member of `oneof`_ ``_last_impression_date_time``.
        end_date_time (str):
            The timestamp of when the campaign will end,
            if applicable. The timestamp is in the
            customer’s timezone and in “yyyy-MM-dd HH:mm:ss”
            format.

            This field is a member of `oneof`_ ``_end_date_time``.
    """

    last_impression_date_time: str = proto.Field(
        proto.STRING,
        number=1,
        optional=True,
    )
    end_date_time: str = proto.Field(
        proto.STRING,
        number=2,
        optional=True,
    )


class SmartCampaignPausedDetails(proto.Message):
    r"""Details related to paused Smart campaigns.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        paused_date_time (str):
            The timestamp of when the campaign was last
            paused. The timestamp is in the customer’s
            timezone and in “yyyy-MM-dd HH:mm:ss” format.

            This field is a member of `oneof`_ ``_paused_date_time``.
    """

    paused_date_time: str = proto.Field(
        proto.STRING,
        number=1,
        optional=True,
    )


class SmartCampaignRemovedDetails(proto.Message):
    r"""Details related to removed Smart campaigns.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        removed_date_time (str):
            The timestamp of when the campaign was
            removed. The timestamp is in the customer’s
            timezone and in “yyyy-MM-dd HH:mm:ss” format.

            This field is a member of `oneof`_ ``_removed_date_time``.
    """

    removed_date_time: str = proto.Field(
        proto.STRING,
        number=1,
        optional=True,
    )


class SmartCampaignEndedDetails(proto.Message):
    r"""Details related to Smart campaigns that have ended.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        end_date_time (str):
            The timestamp of when the campaign ended.
            The timestamp is in the customer’s timezone and
            in “yyyy-MM-dd HH:mm:ss” format.

            This field is a member of `oneof`_ ``_end_date_time``.
    """

    end_date_time: str = proto.Field(
        proto.STRING,
        number=1,
        optional=True,
    )


class GetSmartCampaignStatusResponse(proto.Message):
    r"""Response message for
    [SmartCampaignSettingService.GetSmartCampaignStatus][google.ads.googleads.v20.services.SmartCampaignSettingService.GetSmartCampaignStatus].

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        smart_campaign_status (google.ads.googleads.v20.enums.types.SmartCampaignStatusEnum.SmartCampaignStatus):
            The status of this Smart campaign.
        not_eligible_details (google.ads.googleads.v20.services.types.SmartCampaignNotEligibleDetails):
            Details related to Smart campaigns that are
            ineligible to serve.

            This field is a member of `oneof`_ ``smart_campaign_status_details``.
        eligible_details (google.ads.googleads.v20.services.types.SmartCampaignEligibleDetails):
            Details related to Smart campaigns that are
            eligible to serve.

            This field is a member of `oneof`_ ``smart_campaign_status_details``.
        paused_details (google.ads.googleads.v20.services.types.SmartCampaignPausedDetails):
            Details related to paused Smart campaigns.

            This field is a member of `oneof`_ ``smart_campaign_status_details``.
        removed_details (google.ads.googleads.v20.services.types.SmartCampaignRemovedDetails):
            Details related to removed Smart campaigns.

            This field is a member of `oneof`_ ``smart_campaign_status_details``.
        ended_details (google.ads.googleads.v20.services.types.SmartCampaignEndedDetails):
            Details related to Smart campaigns that have
            ended.

            This field is a member of `oneof`_ ``smart_campaign_status_details``.
    """

    smart_campaign_status: (
        gage_smart_campaign_status.SmartCampaignStatusEnum.SmartCampaignStatus
    ) = proto.Field(
        proto.ENUM,
        number=1,
        enum=gage_smart_campaign_status.SmartCampaignStatusEnum.SmartCampaignStatus,
    )
    not_eligible_details: "SmartCampaignNotEligibleDetails" = proto.Field(
        proto.MESSAGE,
        number=2,
        oneof="smart_campaign_status_details",
        message="SmartCampaignNotEligibleDetails",
    )
    eligible_details: "SmartCampaignEligibleDetails" = proto.Field(
        proto.MESSAGE,
        number=3,
        oneof="smart_campaign_status_details",
        message="SmartCampaignEligibleDetails",
    )
    paused_details: "SmartCampaignPausedDetails" = proto.Field(
        proto.MESSAGE,
        number=4,
        oneof="smart_campaign_status_details",
        message="SmartCampaignPausedDetails",
    )
    removed_details: "SmartCampaignRemovedDetails" = proto.Field(
        proto.MESSAGE,
        number=5,
        oneof="smart_campaign_status_details",
        message="SmartCampaignRemovedDetails",
    )
    ended_details: "SmartCampaignEndedDetails" = proto.Field(
        proto.MESSAGE,
        number=6,
        oneof="smart_campaign_status_details",
        message="SmartCampaignEndedDetails",
    )


class MutateSmartCampaignSettingsRequest(proto.Message):
    r"""Request message for
    [SmartCampaignSettingService.MutateSmartCampaignSettings][google.ads.googleads.v20.services.SmartCampaignSettingService.MutateSmartCampaignSettings].

    Attributes:
        customer_id (str):
            Required. The ID of the customer whose Smart
            campaign settings are being modified.
        operations (MutableSequence[google.ads.googleads.v20.services.types.SmartCampaignSettingOperation]):
            Required. The list of operations to perform
            on individual Smart campaign settings.
        partial_failure (bool):
            If true, successful operations will be
            carried out and invalid operations will return
            errors. If false, all operations will be carried
            out in one transaction if and only if they are
            all valid. Default is false.
        validate_only (bool):
            If true, the request is validated but not
            executed. Only errors are returned, not results.
        response_content_type (google.ads.googleads.v20.enums.types.ResponseContentTypeEnum.ResponseContentType):
            The response content type setting. Determines
            whether the mutable resource or just the
            resource name should be returned post mutation.
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    operations: MutableSequence["SmartCampaignSettingOperation"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=2,
            message="SmartCampaignSettingOperation",
        )
    )
    partial_failure: bool = proto.Field(
        proto.BOOL,
        number=3,
    )
    validate_only: bool = proto.Field(
        proto.BOOL,
        number=4,
    )
    response_content_type: (
        gage_response_content_type.ResponseContentTypeEnum.ResponseContentType
    ) = proto.Field(
        proto.ENUM,
        number=5,
        enum=gage_response_content_type.ResponseContentTypeEnum.ResponseContentType,
    )


class SmartCampaignSettingOperation(proto.Message):
    r"""A single operation to update Smart campaign settings for a
    campaign.

    Attributes:
        update (google.ads.googleads.v20.resources.types.SmartCampaignSetting):
            Update operation: The Smart campaign setting
            must specify a valid resource name.
        update_mask (google.protobuf.field_mask_pb2.FieldMask):
            FieldMask that determines which resource
            fields are modified in an update.
    """

    update: gagr_smart_campaign_setting.SmartCampaignSetting = proto.Field(
        proto.MESSAGE,
        number=1,
        message=gagr_smart_campaign_setting.SmartCampaignSetting,
    )
    update_mask: field_mask_pb2.FieldMask = proto.Field(
        proto.MESSAGE,
        number=2,
        message=field_mask_pb2.FieldMask,
    )


class MutateSmartCampaignSettingsResponse(proto.Message):
    r"""Response message for campaign mutate.

    Attributes:
        partial_failure_error (google.rpc.status_pb2.Status):
            Errors that pertain to operation failures in the partial
            failure mode. Returned only when partial_failure = true and
            all errors occur inside the operations. If any errors occur
            outside the operations (for example, auth errors), we return
            an RPC level error.
        results (MutableSequence[google.ads.googleads.v20.services.types.MutateSmartCampaignSettingResult]):
            All results for the mutate.
    """

    partial_failure_error: status_pb2.Status = proto.Field(
        proto.MESSAGE,
        number=1,
        message=status_pb2.Status,
    )
    results: MutableSequence["MutateSmartCampaignSettingResult"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=2,
            message="MutateSmartCampaignSettingResult",
        )
    )


class MutateSmartCampaignSettingResult(proto.Message):
    r"""The result for the Smart campaign setting mutate.

    Attributes:
        resource_name (str):
            Returned for successful operations.
        smart_campaign_setting (google.ads.googleads.v20.resources.types.SmartCampaignSetting):
            The mutated Smart campaign setting with only mutable fields
            after mutate. The field will only be returned when
            response_content_type is set to "MUTABLE_RESOURCE".
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    smart_campaign_setting: gagr_smart_campaign_setting.SmartCampaignSetting = (
        proto.Field(
            proto.MESSAGE,
            number=2,
            message=gagr_smart_campaign_setting.SmartCampaignSetting,
        )
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.errors",
    marshal="google.ads.googleads.v20",
    manifest={
        "CustomerErrorEnum",
    },
)


class CustomerErrorEnum(proto.Message):
    r"""Container for enum describing possible customer errors."""

    class CustomerError(proto.Enum):
        r"""Set of errors that are related to requests dealing with
        Customer.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            STATUS_CHANGE_DISALLOWED (2):
                Customer status is not allowed to be changed
                from DRAFT and CLOSED. Currency code and at
                least one of country code and time zone needs to
                be set when status is changed to ENABLED.
            ACCOUNT_NOT_SET_UP (3):
                CustomerService cannot get a customer that
                has not been fully set up.
            CREATION_DENIED_FOR_POLICY_VIOLATION (4):
                Customer creation is denied for policy
                violation.
            CREATION_DENIED_INELIGIBLE_MCC (5):
                Manager account is ineligible to create new
                accounts.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        STATUS_CHANGE_DISALLOWED = 2
        ACCOUNT_NOT_SET_UP = 3
        CREATION_DENIED_FOR_POLICY_VIOLATION = 4
        CREATION_DENIED_INELIGIBLE_MCC = 5


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.enums",
    marshal="google.ads.googleads.v21",
    manifest={
        "LocalServicesLeadCreditIssuanceDecisionEnum",
    },
)


class LocalServicesLeadCreditIssuanceDecisionEnum(proto.Message):
    r"""Container for enum describing possible credit issuance
    decisions for a lead.

    """

    class CreditIssuanceDecision(proto.Enum):
        r"""Decision of bonus credit issued or rejected.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            SUCCESS_NOT_REACHED_THRESHOLD (2):
                Bonus credit is issued successfully and bonus
                credit cap has not reached the threshold after
                issuing this bonus credit.
            SUCCESS_REACHED_THRESHOLD (3):
                Bonus credit is issued successfully and bonus
                credit cap has reached the threshold after
                issuing this bonus credit.
            FAIL_OVER_THRESHOLD (4):
                Bonus credit is not issued because the
                provider has reached the bonus credit cap.
            FAIL_NOT_ELIGIBLE (5):
                Bonus credit is not issued because this lead
                is not eligible for bonus credit.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        SUCCESS_NOT_REACHED_THRESHOLD = 2
        SUCCESS_REACHED_THRESHOLD = 3
        FAIL_OVER_THRESHOLD = 4
        FAIL_NOT_ELIGIBLE = 5


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.errors",
    marshal="google.ads.googleads.v19",
    manifest={
        "SizeLimitErrorEnum",
    },
)


class SizeLimitErrorEnum(proto.Message):
    r"""Container for enum describing possible size limit errors."""

    class SizeLimitError(proto.Enum):
        r"""Enum describing possible size limit errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            REQUEST_SIZE_LIMIT_EXCEEDED (2):
                The number of entries in the request exceeds
                the system limit, or the contents of the
                operations exceed transaction limits due to
                their size or complexity. Try reducing the
                number of entries per request.
            RESPONSE_SIZE_LIMIT_EXCEEDED (3):
                The number of entries in the response exceeds
                the system limit.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        REQUEST_SIZE_LIMIT_EXCEEDED = 2
        RESPONSE_SIZE_LIMIT_EXCEEDED = 3


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v19.common.types import policy
from google.ads.googleads.v19.enums.types import (
    asset_field_type as gage_asset_field_type,
)
from google.ads.googleads.v19.enums.types import (
    response_content_type as gage_response_content_type,
)
from google.ads.googleads.v19.resources.types import (
    ad_group_ad as gagr_ad_group_ad,
)
from google.protobuf import field_mask_pb2  # type: ignore
from google.rpc import status_pb2  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.services",
    marshal="google.ads.googleads.v19",
    manifest={
        "MutateAdGroupAdsRequest",
        "AdGroupAdOperation",
        "MutateAdGroupAdsResponse",
        "MutateAdGroupAdResult",
        "RemoveAutomaticallyCreatedAssetsRequest",
        "AssetsWithFieldType",
    },
)


class MutateAdGroupAdsRequest(proto.Message):
    r"""Request message for
    [AdGroupAdService.MutateAdGroupAds][google.ads.googleads.v19.services.AdGroupAdService.MutateAdGroupAds].

    Attributes:
        customer_id (str):
            Required. The ID of the customer whose ads
            are being modified.
        operations (MutableSequence[google.ads.googleads.v19.services.types.AdGroupAdOperation]):
            Required. The list of operations to perform
            on individual ads.
        partial_failure (bool):
            If true, successful operations will be
            carried out and invalid operations will return
            errors. If false, all operations will be carried
            out in one transaction if and only if they are
            all valid. Default is false.
        validate_only (bool):
            If true, the request is validated but not
            executed. Only errors are returned, not results.
        response_content_type (google.ads.googleads.v19.enums.types.ResponseContentTypeEnum.ResponseContentType):
            The response content type setting. Determines
            whether the mutable resource or just the
            resource name should be returned post mutation.
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    operations: MutableSequence["AdGroupAdOperation"] = proto.RepeatedField(
        proto.MESSAGE,
        number=2,
        message="AdGroupAdOperation",
    )
    partial_failure: bool = proto.Field(
        proto.BOOL,
        number=3,
    )
    validate_only: bool = proto.Field(
        proto.BOOL,
        number=4,
    )
    response_content_type: (
        gage_response_content_type.ResponseContentTypeEnum.ResponseContentType
    ) = proto.Field(
        proto.ENUM,
        number=5,
        enum=gage_response_content_type.ResponseContentTypeEnum.ResponseContentType,
    )


class AdGroupAdOperation(proto.Message):
    r"""A single operation (create, update, remove) on an ad group
    ad.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        update_mask (google.protobuf.field_mask_pb2.FieldMask):
            FieldMask that determines which resource
            fields are modified in an update.
        policy_validation_parameter (google.ads.googleads.v19.common.types.PolicyValidationParameter):
            Configuration for how policies are validated.
        create (google.ads.googleads.v19.resources.types.AdGroupAd):
            Create operation: No resource name is
            expected for the new ad.

            This field is a member of `oneof`_ ``operation``.
        update (google.ads.googleads.v19.resources.types.AdGroupAd):
            Update operation: The ad is expected to have
            a valid resource name.

            This field is a member of `oneof`_ ``operation``.
        remove (str):
            Remove operation: A resource name for the removed ad is
            expected, in this format:

            ``customers/{customer_id}/adGroupAds/{ad_group_id}~{ad_id}``

            This field is a member of `oneof`_ ``operation``.
    """

    update_mask: field_mask_pb2.FieldMask = proto.Field(
        proto.MESSAGE,
        number=4,
        message=field_mask_pb2.FieldMask,
    )
    policy_validation_parameter: policy.PolicyValidationParameter = proto.Field(
        proto.MESSAGE,
        number=5,
        message=policy.PolicyValidationParameter,
    )
    create: gagr_ad_group_ad.AdGroupAd = proto.Field(
        proto.MESSAGE,
        number=1,
        oneof="operation",
        message=gagr_ad_group_ad.AdGroupAd,
    )
    update: gagr_ad_group_ad.AdGroupAd = proto.Field(
        proto.MESSAGE,
        number=2,
        oneof="operation",
        message=gagr_ad_group_ad.AdGroupAd,
    )
    remove: str = proto.Field(
        proto.STRING,
        number=3,
        oneof="operation",
    )


class MutateAdGroupAdsResponse(proto.Message):
    r"""Response message for an ad group ad mutate.

    Attributes:
        partial_failure_error (google.rpc.status_pb2.Status):
            Errors that pertain to operation failures in the partial
            failure mode. Returned only when partial_failure = true and
            all errors occur inside the operations. If any errors occur
            outside the operations (for example, auth errors), we return
            an RPC level error.
        results (MutableSequence[google.ads.googleads.v19.services.types.MutateAdGroupAdResult]):
            All results for the mutate.
    """

    partial_failure_error: status_pb2.Status = proto.Field(
        proto.MESSAGE,
        number=3,
        message=status_pb2.Status,
    )
    results: MutableSequence["MutateAdGroupAdResult"] = proto.RepeatedField(
        proto.MESSAGE,
        number=2,
        message="MutateAdGroupAdResult",
    )


class MutateAdGroupAdResult(proto.Message):
    r"""The result for the ad mutate.

    Attributes:
        resource_name (str):
            The resource name returned for successful
            operations.
        ad_group_ad (google.ads.googleads.v19.resources.types.AdGroupAd):
            The mutated ad group ad with only mutable fields after
            mutate. The field will only be returned when
            response_content_type is set to "MUTABLE_RESOURCE".
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    ad_group_ad: gagr_ad_group_ad.AdGroupAd = proto.Field(
        proto.MESSAGE,
        number=2,
        message=gagr_ad_group_ad.AdGroupAd,
    )


class RemoveAutomaticallyCreatedAssetsRequest(proto.Message):
    r"""Request message for
    [AdGroupAdService.RemoveAutomaticallyCreatedAssetsRequest][].

    Attributes:
        ad_group_ad (str):
            Required. The resource name of the AdGroupAd
            from which to remove automatically created
            assets.
        assets_with_field_type (MutableSequence[google.ads.googleads.v19.services.types.AssetsWithFieldType]):
            Required. List of assets with field type to
            be removed from the AdGroupAd.
    """

    ad_group_ad: str = proto.Field(
        proto.STRING,
        number=1,
    )
    assets_with_field_type: MutableSequence["AssetsWithFieldType"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=2,
            message="AssetsWithFieldType",
        )
    )


class AssetsWithFieldType(proto.Message):
    r"""The combination of system asset and field type to remove.

    Attributes:
        asset (str):
            Required. The resource name of the asset to
            be removed.
        asset_field_type (google.ads.googleads.v19.enums.types.AssetFieldTypeEnum.AssetFieldType):
            Required. The asset field type.
    """

    asset: str = proto.Field(
        proto.STRING,
        number=1,
    )
    asset_field_type: (
        gage_asset_field_type.AssetFieldTypeEnum.AssetFieldType
    ) = proto.Field(
        proto.ENUM,
        number=2,
        enum=gage_asset_field_type.AssetFieldTypeEnum.AssetFieldType,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.errors",
    marshal="google.ads.googleads.v20",
    manifest={
        "AssetSetLinkErrorEnum",
    },
)


class AssetSetLinkErrorEnum(proto.Message):
    r"""Container for enum describing possible asset set link errors."""

    class AssetSetLinkError(proto.Enum):
        r"""Enum describing possible asset set link errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            INCOMPATIBLE_ADVERTISING_CHANNEL_TYPE (2):
                Advertising channel type cannot be attached
                to the asset set due to channel-based
                restrictions.
            DUPLICATE_FEED_LINK (3):
                For this asset set type, only one campaign to
                feed linkage is allowed.
            INCOMPATIBLE_ASSET_SET_TYPE_WITH_CAMPAIGN_TYPE (4):
                The asset set type and campaign type are
                incompatible.
            DUPLICATE_ASSET_SET_LINK (5):
                Cannot link duplicate asset sets to the same
                campaign.
            ASSET_SET_LINK_CANNOT_BE_REMOVED (6):
                Cannot remove the asset set link. If a
                campaign is linked with only one asset set and
                you attempt to unlink them, this error will be
                triggered.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        INCOMPATIBLE_ADVERTISING_CHANNEL_TYPE = 2
        DUPLICATE_FEED_LINK = 3
        INCOMPATIBLE_ASSET_SET_TYPE_WITH_CAMPAIGN_TYPE = 4
        DUPLICATE_ASSET_SET_LINK = 5
        ASSET_SET_LINK_CANNOT_BE_REMOVED = 6


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v21.resources.types import (
    customer_sk_ad_network_conversion_value_schema,
)
from google.rpc import status_pb2  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.services",
    marshal="google.ads.googleads.v21",
    manifest={
        "CustomerSkAdNetworkConversionValueSchemaOperation",
        "MutateCustomerSkAdNetworkConversionValueSchemaRequest",
        "MutateCustomerSkAdNetworkConversionValueSchemaResult",
        "MutateCustomerSkAdNetworkConversionValueSchemaResponse",
    },
)


class CustomerSkAdNetworkConversionValueSchemaOperation(proto.Message):
    r"""A single update operation for a
    CustomerSkAdNetworkConversionValueSchema.

    Attributes:
        update (google.ads.googleads.v21.resources.types.CustomerSkAdNetworkConversionValueSchema):
            Update operation: The schema is expected to
            have a valid resource name.
    """

    update: (
        customer_sk_ad_network_conversion_value_schema.CustomerSkAdNetworkConversionValueSchema
    ) = proto.Field(
        proto.MESSAGE,
        number=1,
        message=customer_sk_ad_network_conversion_value_schema.CustomerSkAdNetworkConversionValueSchema,
    )


class MutateCustomerSkAdNetworkConversionValueSchemaRequest(proto.Message):
    r"""Request message for
    [CustomerSkAdNetworkConversionValueSchemaService.MutateCustomerSkAdNetworkConversionValueSchema][google.ads.googleads.v21.services.CustomerSkAdNetworkConversionValueSchemaService.MutateCustomerSkAdNetworkConversionValueSchema].

    Attributes:
        customer_id (str):
            The ID of the customer whose shared sets are
            being modified.
        operation (google.ads.googleads.v21.services.types.CustomerSkAdNetworkConversionValueSchemaOperation):
            The operation to perform.
        validate_only (bool):
            If true, the request is validated but not
            executed. Only errors are returned, not results.
        enable_warnings (bool):
            Optional. If true, enables returning
            warnings. Warnings return error messages and
            error codes without blocking the execution of
            the mutate operation.
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    operation: "CustomerSkAdNetworkConversionValueSchemaOperation" = (
        proto.Field(
            proto.MESSAGE,
            number=2,
            message="CustomerSkAdNetworkConversionValueSchemaOperation",
        )
    )
    validate_only: bool = proto.Field(
        proto.BOOL,
        number=3,
    )
    enable_warnings: bool = proto.Field(
        proto.BOOL,
        number=4,
    )


class MutateCustomerSkAdNetworkConversionValueSchemaResult(proto.Message):
    r"""The result for the CustomerSkAdNetworkConversionValueSchema
    mutate.

    Attributes:
        resource_name (str):
            Resource name of the customer that was
            modified.
        app_id (str):
            App ID of the SkanConversionValue modified.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    app_id: str = proto.Field(
        proto.STRING,
        number=2,
    )


class MutateCustomerSkAdNetworkConversionValueSchemaResponse(proto.Message):
    r"""Response message for
    MutateCustomerSkAdNetworkConversionValueSchema.

    Attributes:
        result (google.ads.googleads.v21.services.types.MutateCustomerSkAdNetworkConversionValueSchemaResult):
            All results for the mutate.
        warning (google.rpc.status_pb2.Status):
            Non blocking errors that provides schema validation failure
            details. Returned only when enable_warnings = true.
    """

    result: "MutateCustomerSkAdNetworkConversionValueSchemaResult" = (
        proto.Field(
            proto.MESSAGE,
            number=1,
            message="MutateCustomerSkAdNetworkConversionValueSchemaResult",
        )
    )
    warning: status_pb2.Status = proto.Field(
        proto.MESSAGE,
        number=2,
        message=status_pb2.Status,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from collections import OrderedDict
from http import HTTPStatus
import json
import logging as std_logging
import os
import re
from typing import (
    Dict,
    Callable,
    MutableSequence,
    Optional,
    Sequence,
    Tuple,
    Type,
    Union,
    cast,
)
import warnings

from google.ads.googleads.v19 import gapic_version as package_version

from google.api_core import client_options as client_options_lib
from google.api_core import exceptions as core_exceptions
from google.api_core import gapic_v1
from google.api_core import retry as retries
from google.auth import credentials as ga_credentials  # type: ignore
from google.auth.transport import mtls  # type: ignore
from google.auth.transport.grpc import SslCredentials  # type: ignore
from google.auth.exceptions import MutualTLSChannelError  # type: ignore
from google.oauth2 import service_account  # type: ignore
import google.protobuf

try:
    OptionalRetry = Union[retries.Retry, gapic_v1.method._MethodDefault, None]
except AttributeError:  # pragma: NO COVER
    OptionalRetry = Union[retries.Retry, object, None]  # type: ignore

try:
    from google.api_core import client_logging  # type: ignore

    CLIENT_LOGGING_SUPPORTED = True  # pragma: NO COVER
except ImportError:  # pragma: NO COVER
    CLIENT_LOGGING_SUPPORTED = False

_LOGGER = std_logging.getLogger(__name__)

from google.ads.googleads.v19.resources.types import offline_user_data_job
from google.ads.googleads.v19.services.types import (
    offline_user_data_job_service,
)
from google.api_core import operation  # type: ignore
from google.api_core import operation_async  # type: ignore
from google.protobuf import empty_pb2  # type: ignore
from google.rpc import status_pb2  # type: ignore
from .transports.base import (
    OfflineUserDataJobServiceTransport,
    DEFAULT_CLIENT_INFO,
)
from .transports.grpc import OfflineUserDataJobServiceGrpcTransport
from .transports.grpc_asyncio import (
    OfflineUserDataJobServiceGrpcAsyncIOTransport,
)


class OfflineUserDataJobServiceClientMeta(type):
    """Metaclass for the OfflineUserDataJobService client.

    This provides class-level methods for building and retrieving
    support objects (e.g. transport) without polluting the client instance
    objects.
    """

    _transport_registry = (
        OrderedDict()
    )  # type: Dict[str, Type[OfflineUserDataJobServiceTransport]]
    _transport_registry["grpc"] = OfflineUserDataJobServiceGrpcTransport
    _transport_registry["grpc_asyncio"] = (
        OfflineUserDataJobServiceGrpcAsyncIOTransport
    )

    def get_transport_class(
        cls,
        label: Optional[str] = None,
    ) -> Type[OfflineUserDataJobServiceTransport]:
        """Returns an appropriate transport class.

        Args:
            label: The name of the desired transport. If none is
                provided, then the first transport in the registry is used.

        Returns:
            The transport class to use.
        """
        # If a specific transport is requested, return that one.
        if label:
            return cls._transport_registry[label]

        # No transport is requested; return the default (that is, the first one
        # in the dictionary).
        return next(iter(cls._transport_registry.values()))


class OfflineUserDataJobServiceClient(
    metaclass=OfflineUserDataJobServiceClientMeta
):
    """Service to manage offline user data jobs."""

    @staticmethod
    def _get_default_mtls_endpoint(api_endpoint):
        """Converts api endpoint to mTLS endpoint.

        Convert "*.sandbox.googleapis.com" and "*.googleapis.com" to
        "*.mtls.sandbox.googleapis.com" and "*.mtls.googleapis.com" respectively.
        Args:
            api_endpoint (Optional[str]): the api endpoint to convert.
        Returns:
            str: converted mTLS api endpoint.
        """
        if not api_endpoint:
            return api_endpoint

        mtls_endpoint_re = re.compile(
            r"(?P<name>[^.]+)(?P<mtls>\.mtls)?(?P<sandbox>\.sandbox)?(?P<googledomain>\.googleapis\.com)?"
        )

        m = mtls_endpoint_re.match(api_endpoint)
        name, mtls, sandbox, googledomain = m.groups()
        if mtls or not googledomain:
            return api_endpoint

        if sandbox:
            return api_endpoint.replace(
                "sandbox.googleapis.com", "mtls.sandbox.googleapis.com"
            )

        return api_endpoint.replace(".googleapis.com", ".mtls.googleapis.com")

    # Note: DEFAULT_ENDPOINT is deprecated. Use _DEFAULT_ENDPOINT_TEMPLATE instead.
    DEFAULT_ENDPOINT = "googleads.googleapis.com"
    DEFAULT_MTLS_ENDPOINT = _get_default_mtls_endpoint.__func__(  # type: ignore
        DEFAULT_ENDPOINT
    )

    _DEFAULT_ENDPOINT_TEMPLATE = "googleads.{UNIVERSE_DOMAIN}"
    _DEFAULT_UNIVERSE = "googleapis.com"

    @classmethod
    def from_service_account_info(cls, info: dict, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            info.

        Args:
            info (dict): The service account private key info.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            OfflineUserDataJobServiceClient: The constructed client.
        """
        credentials = service_account.Credentials.from_service_account_info(
            info
        )
        kwargs["credentials"] = credentials
        return cls(*args, **kwargs)

    @classmethod
    def from_service_account_file(cls, filename: str, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            file.

        Args:
            filename (str): The path to the service account private key json
                file.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            OfflineUserDataJobServiceClient: The constructed client.
        """
        credentials = service_account.Credentials.from_service_account_file(
            filename
        )
        kwargs["credentials"] = credentials
        return cls(*args, **kwargs)

    from_service_account_json = from_service_account_file

    @property
    def transport(self) -> OfflineUserDataJobServiceTransport:
        """Returns the transport used by the client instance.

        Returns:
            OfflineUserDataJobServiceTransport: The transport used by the client
                instance.
        """
        return self._transport

    @staticmethod
    def offline_user_data_job_path(
        customer_id: str,
        offline_user_data_update_id: str,
    ) -> str:
        """Returns a fully-qualified offline_user_data_job string."""
        return "customers/{customer_id}/offlineUserDataJobs/{offline_user_data_update_id}".format(
            customer_id=customer_id,
            offline_user_data_update_id=offline_user_data_update_id,
        )

    @staticmethod
    def parse_offline_user_data_job_path(path: str) -> Dict[str, str]:
        """Parses a offline_user_data_job path into its component segments."""
        m = re.match(
            r"^customers/(?P<customer_id>.+?)/offlineUserDataJobs/(?P<offline_user_data_update_id>.+?)$",
            path,
        )
        return m.groupdict() if m else {}

    @staticmethod
    def common_billing_account_path(
        billing_account: str,
    ) -> str:
        """Returns a fully-qualified billing_account string."""
        return "billingAccounts/{billing_account}".format(
            billing_account=billing_account,
        )

    @staticmethod
    def parse_common_billing_account_path(path: str) -> Dict[str, str]:
        """Parse a billing_account path into its component segments."""
        m = re.match(r"^billingAccounts/(?P<billing_account>.+?)$", path)
        return m.groupdict() if m else {}

    @staticmethod
    def common_folder_path(
        folder: str,
    ) -> str:
        """Returns a fully-qualified folder string."""
        return "folders/{folder}".format(
            folder=folder,
        )

    @staticmethod
    def parse_common_folder_path(path: str) -> Dict[str, str]:
        """Parse a folder path into its component segments."""
        m = re.match(r"^folders/(?P<folder>.+?)$", path)
        return m.groupdict() if m else {}

    @staticmethod
    def common_organization_path(
        organization: str,
    ) -> str:
        """Returns a fully-qualified organization string."""
        return "organizations/{organization}".format(
            organization=organization,
        )

    @staticmethod
    def parse_common_organization_path(path: str) -> Dict[str, str]:
        """Parse a organization path into its component segments."""
        m = re.match(r"^organizations/(?P<organization>.+?)$", path)
        return m.groupdict() if m else {}

    @staticmethod
    def common_project_path(
        project: str,
    ) -> str:
        """Returns a fully-qualified project string."""
        return "projects/{project}".format(
            project=project,
        )

    @staticmethod
    def parse_common_project_path(path: str) -> Dict[str, str]:
        """Parse a project path into its component segments."""
        m = re.match(r"^projects/(?P<project>.+?)$", path)
        return m.groupdict() if m else {}

    @staticmethod
    def common_location_path(
        project: str,
        location: str,
    ) -> str:
        """Returns a fully-qualified location string."""
        return "projects/{project}/locations/{location}".format(
            project=project,
            location=location,
        )

    @staticmethod
    def parse_common_location_path(path: str) -> Dict[str, str]:
        """Parse a location path into its component segments."""
        m = re.match(
            r"^projects/(?P<project>.+?)/locations/(?P<location>.+?)$", path
        )
        return m.groupdict() if m else {}

    @classmethod
    def get_mtls_endpoint_and_cert_source(
        cls, client_options: Optional[client_options_lib.ClientOptions] = None
    ):
        """Deprecated. Return the API endpoint and client cert source for mutual TLS.

        The client cert source is determined in the following order:
        (1) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is not "true", the
        client cert source is None.
        (2) if `client_options.client_cert_source` is provided, use the provided one; if the
        default client cert source exists, use the default one; otherwise the client cert
        source is None.

        The API endpoint is determined in the following order:
        (1) if `client_options.api_endpoint` if provided, use the provided one.
        (2) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is "always", use the
        default mTLS endpoint; if the environment variable is "never", use the default API
        endpoint; otherwise if client cert source exists, use the default mTLS endpoint, otherwise
        use the default API endpoint.

        More details can be found at https://google.aip.dev/auth/4114.

        Args:
            client_options (google.api_core.client_options.ClientOptions): Custom options for the
                client. Only the `api_endpoint` and `client_cert_source` properties may be used
                in this method.

        Returns:
            Tuple[str, Callable[[], Tuple[bytes, bytes]]]: returns the API endpoint and the
                client cert source to use.

        Raises:
            google.auth.exceptions.MutualTLSChannelError: If any errors happen.
        """

        warnings.warn(
            "get_mtls_endpoint_and_cert_source is deprecated. Use the api_endpoint property instead.",
            DeprecationWarning,
        )
        if client_options is None:
            client_options = client_options_lib.ClientOptions()
        use_client_cert = os.getenv(
            "GOOGLE_API_USE_CLIENT_CERTIFICATE", "false"
        )
        use_mtls_endpoint = os.getenv("GOOGLE_API_USE_MTLS_ENDPOINT", "auto")
        if use_client_cert not in ("true", "false"):
            raise ValueError(
                "Environment variable `GOOGLE_API_USE_CLIENT_CERTIFICATE` must be either `true` or `false`"
            )
        if use_mtls_endpoint not in ("auto", "never", "always"):
            raise MutualTLSChannelError(
                "Environment variable `GOOGLE_API_USE_MTLS_ENDPOINT` must be `never`, `auto` or `always`"
            )

        # Figure out the client cert source to use.
        client_cert_source = None
        if use_client_cert == "true":
            if client_options.client_cert_source:
                client_cert_source = client_options.client_cert_source
            elif mtls.has_default_client_cert_source():
                client_cert_source = mtls.default_client_cert_source()

        # Figure out which api endpoint to use.
        if client_options.api_endpoint is not None:
            api_endpoint = client_options.api_endpoint
        elif use_mtls_endpoint == "always" or (
            use_mtls_endpoint == "auto" and client_cert_source
        ):
            api_endpoint = cls.DEFAULT_MTLS_ENDPOINT
        else:
            api_endpoint = cls.DEFAULT_ENDPOINT

        return api_endpoint, client_cert_source

    @staticmethod
    def _read_environment_variables():
        """Returns the environment variables used by the client.

        Returns:
            Tuple[bool, str, str]: returns the GOOGLE_API_USE_CLIENT_CERTIFICATE,
            GOOGLE_API_USE_MTLS_ENDPOINT, and GOOGLE_CLOUD_UNIVERSE_DOMAIN environment variables.

        Raises:
            ValueError: If GOOGLE_API_USE_CLIENT_CERTIFICATE is not
                any of ["true", "false"].
            google.auth.exceptions.MutualTLSChannelError: If GOOGLE_API_USE_MTLS_ENDPOINT
                is not any of ["auto", "never", "always"].
        """
        use_client_cert = os.getenv(
            "GOOGLE_API_USE_CLIENT_CERTIFICATE", "false"
        ).lower()
        use_mtls_endpoint = os.getenv(
            "GOOGLE_API_USE_MTLS_ENDPOINT", "auto"
        ).lower()
        universe_domain_env = os.getenv("GOOGLE_CLOUD_UNIVERSE_DOMAIN")
        if use_client_cert not in ("true", "false"):
            raise ValueError(
                "Environment variable `GOOGLE_API_USE_CLIENT_CERTIFICATE` must be either `true` or `false`"
            )
        if use_mtls_endpoint not in ("auto", "never", "always"):
            raise MutualTLSChannelError(
                "Environment variable `GOOGLE_API_USE_MTLS_ENDPOINT` must be `never`, `auto` or `always`"
            )
        return use_client_cert == "true", use_mtls_endpoint, universe_domain_env

    @staticmethod
    def _get_client_cert_source(provided_cert_source, use_cert_flag):
        """Return the client cert source to be used by the client.

        Args:
            provided_cert_source (bytes): The client certificate source provided.
            use_cert_flag (bool): A flag indicating whether to use the client certificate.

        Returns:
            bytes or None: The client cert source to be used by the client.
        """
        client_cert_source = None
        if use_cert_flag:
            if provided_cert_source:
                client_cert_source = provided_cert_source
            elif mtls.has_default_client_cert_source():
                client_cert_source = mtls.default_client_cert_source()
        return client_cert_source

    @staticmethod
    def _get_api_endpoint(
        api_override, client_cert_source, universe_domain, use_mtls_endpoint
    ):
        """Return the API endpoint used by the client.

        Args:
            api_override (str): The API endpoint override. If specified, this is always
                the return value of this function and the other arguments are not used.
            client_cert_source (bytes): The client certificate source used by the client.
            universe_domain (str): The universe domain used by the client.
            use_mtls_endpoint (str): How to use the mTLS endpoint, which depends also on the other parameters.
                Possible values are "always", "auto", or "never".

        Returns:
            str: The API endpoint to be used by the client.
        """
        if api_override is not None:
            api_endpoint = api_override
        elif use_mtls_endpoint == "always" or (
            use_mtls_endpoint == "auto" and client_cert_source
        ):
            _default_universe = (
                OfflineUserDataJobServiceClient._DEFAULT_UNIVERSE
            )
            if universe_domain != _default_universe:
                raise MutualTLSChannelError(
                    f"mTLS is not supported in any universe other than {_default_universe}."
                )
            api_endpoint = OfflineUserDataJobServiceClient.DEFAULT_MTLS_ENDPOINT
        else:
            api_endpoint = OfflineUserDataJobServiceClient._DEFAULT_ENDPOINT_TEMPLATE.format(
                UNIVERSE_DOMAIN=universe_domain
            )
        return api_endpoint

    @staticmethod
    def _get_universe_domain(
        client_universe_domain: Optional[str],
        universe_domain_env: Optional[str],
    ) -> str:
        """Return the universe domain used by the client.

        Args:
            client_universe_domain (Optional[str]): The universe domain configured via the client options.
            universe_domain_env (Optional[str]): The universe domain configured via the "GOOGLE_CLOUD_UNIVERSE_DOMAIN" environment variable.

        Returns:
            str: The universe domain to be used by the client.

        Raises:
            ValueError: If the universe domain is an empty string.
        """
        universe_domain = OfflineUserDataJobServiceClient._DEFAULT_UNIVERSE
        if client_universe_domain is not None:
            universe_domain = client_universe_domain
        elif universe_domain_env is not None:
            universe_domain = universe_domain_env
        if len(universe_domain.strip()) == 0:
            raise ValueError("Universe Domain cannot be an empty string.")
        return universe_domain

    def _validate_universe_domain(self):
        """Validates client's and credentials' universe domains are consistent.

        Returns:
            bool: True iff the configured universe domain is valid.

        Raises:
            ValueError: If the configured universe domain is not valid.
        """

        # NOTE (b/349488459): universe validation is disabled until further notice.
        return True

    def _add_cred_info_for_auth_errors(
        self, error: core_exceptions.GoogleAPICallError
    ) -> None:
        """Adds credential info string to error details for 401/403/404 errors.

        Args:
            error (google.api_core.exceptions.GoogleAPICallError): The error to add the cred info.
        """
        if error.code not in [
            HTTPStatus.UNAUTHORIZED,
            HTTPStatus.FORBIDDEN,
            HTTPStatus.NOT_FOUND,
        ]:
            return

        cred = self._transport._credentials

        # get_cred_info is only available in google-auth>=2.35.0
        if not hasattr(cred, "get_cred_info"):
            return

        # ignore the type check since pypy test fails when get_cred_info
        # is not available
        cred_info = cred.get_cred_info()  # type: ignore
        if cred_info and hasattr(error._details, "append"):
            error._details.append(json.dumps(cred_info))

    @property
    def api_endpoint(self):
        """Return the API endpoint used by the client instance.

        Returns:
            str: The API endpoint used by the client instance.
        """
        return self._api_endpoint

    @property
    def universe_domain(self) -> str:
        """Return the universe domain used by the client instance.

        Returns:
            str: The universe domain used by the client instance.
        """
        return self._universe_domain

    def __init__(
        self,
        *,
        credentials: Optional[ga_credentials.Credentials] = None,
        transport: Optional[
            Union[
                str,
                OfflineUserDataJobServiceTransport,
                Callable[..., OfflineUserDataJobServiceTransport],
            ]
        ] = None,
        client_options: Optional[
            Union[client_options_lib.ClientOptions, dict]
        ] = None,
        client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
    ) -> None:
        """Instantiates the offline user data job service client.

        Args:
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.
            transport (Optional[Union[str,OfflineUserDataJobServiceTransport,Callable[..., OfflineUserDataJobServiceTransport]]]):
                The transport to use, or a Callable that constructs and returns a new transport.
                If a Callable is given, it will be called with the same set of initialization
                arguments as used in the OfflineUserDataJobServiceTransport constructor.
                If set to None, a transport is chosen automatically.
            client_options (Optional[Union[google.api_core.client_options.ClientOptions, dict]]):
                Custom options for the client.

                1. The ``api_endpoint`` property can be used to override the
                default endpoint provided by the client when ``transport`` is
                not explicitly provided. Only if this property is not set and
                ``transport`` was not explicitly provided, the endpoint is
                determined by the GOOGLE_API_USE_MTLS_ENDPOINT environment
                variable, which have one of the following values:
                "always" (always use the default mTLS endpoint), "never" (always
                use the default regular endpoint) and "auto" (auto-switch to the
                default mTLS endpoint if client certificate is present; this is
                the default value).

                2. If the GOOGLE_API_USE_CLIENT_CERTIFICATE environment variable
                is "true", then the ``client_cert_source`` property can be used
                to provide a client certificate for mTLS transport. If
                not provided, the default SSL client certificate will be used if
                present. If GOOGLE_API_USE_CLIENT_CERTIFICATE is "false" or not
                set, no client certificate will be used.

                3. The ``universe_domain`` property can be used to override the
                default "googleapis.com" universe. Note that the ``api_endpoint``
                property still takes precedence; and ``universe_domain`` is
                currently not supported for mTLS.

            client_info (google.api_core.gapic_v1.client_info.ClientInfo):
                The client info used to send a user-agent string along with
                API requests. If ``None``, then default info will be used.
                Generally, you only need to set this if you're developing
                your own client library.

        Raises:
            google.auth.exceptions.MutualTLSChannelError: If mutual TLS transport
                creation failed for any reason.
        """
        self._client_options = client_options
        if isinstance(self._client_options, dict):
            self._client_options = client_options_lib.from_dict(
                self._client_options
            )
        if self._client_options is None:
            self._client_options = client_options_lib.ClientOptions()
        self._client_options = cast(
            client_options_lib.ClientOptions, self._client_options
        )

        universe_domain_opt = getattr(
            self._client_options, "universe_domain", None
        )

        (
            self._use_client_cert,
            self._use_mtls_endpoint,
            self._universe_domain_env,
        ) = OfflineUserDataJobServiceClient._read_environment_variables()
        self._client_cert_source = (
            OfflineUserDataJobServiceClient._get_client_cert_source(
                self._client_options.client_cert_source, self._use_client_cert
            )
        )
        self._universe_domain = (
            OfflineUserDataJobServiceClient._get_universe_domain(
                universe_domain_opt, self._universe_domain_env
            )
        )
        self._api_endpoint = None  # updated below, depending on `transport`

        # Initialize the universe domain validation.
        self._is_universe_domain_valid = False

        if CLIENT_LOGGING_SUPPORTED:  # pragma: NO COVER
            # Setup logging.
            client_logging.initialize_logging()

        api_key_value = getattr(self._client_options, "api_key", None)
        if api_key_value and credentials:
            raise ValueError(
                "client_options.api_key and credentials are mutually exclusive"
            )

        # Save or instantiate the transport.
        # Ordinarily, we provide the transport, but allowing a custom transport
        # instance provides an extensibility point for unusual situations.
        transport_provided = isinstance(
            transport, OfflineUserDataJobServiceTransport
        )
        if transport_provided:
            # transport is a OfflineUserDataJobServiceTransport instance.
            if (
                credentials
                or self._client_options.credentials_file
                or api_key_value
            ):
                raise ValueError(
                    "When providing a transport instance, "
                    "provide its credentials directly."
                )
            if self._client_options.scopes:
                raise ValueError(
                    "When providing a transport instance, provide its scopes "
                    "directly."
                )
            self._transport = cast(
                OfflineUserDataJobServiceTransport, transport
            )
            self._api_endpoint = self._transport.host

        self._api_endpoint = (
            self._api_endpoint
            or OfflineUserDataJobServiceClient._get_api_endpoint(
                self._client_options.api_endpoint,
                self._client_cert_source,
                self._universe_domain,
                self._use_mtls_endpoint,
            )
        )

        if not transport_provided:
            import google.auth._default  # type: ignore

            if api_key_value and hasattr(
                google.auth._default, "get_api_key_credentials"
            ):
                credentials = google.auth._default.get_api_key_credentials(
                    api_key_value
                )

            transport_init: Union[
                Type[OfflineUserDataJobServiceTransport],
                Callable[..., OfflineUserDataJobServiceTransport],
            ] = (
                OfflineUserDataJobServiceClient.get_transport_class(transport)
                if isinstance(transport, str) or transport is None
                else cast(
                    Callable[..., OfflineUserDataJobServiceTransport], transport
                )
            )
            # initialize with the provided callable or the passed in class
            self._transport = transport_init(
                credentials=credentials,
                credentials_file=self._client_options.credentials_file,
                host=self._api_endpoint,
                scopes=self._client_options.scopes,
                client_cert_source_for_mtls=self._client_cert_source,
                quota_project_id=self._client_options.quota_project_id,
                client_info=client_info,
                always_use_jwt_access=True,
                api_audience=self._client_options.api_audience,
            )

        if "async" not in str(self._transport):
            if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(
                std_logging.DEBUG
            ):  # pragma: NO COVER
                _LOGGER.debug(
                    "Created client `google.ads.googleads.v19.services.OfflineUserDataJobServiceClient`.",
                    extra=(
                        {
                            "serviceName": "google.ads.googleads.v19.services.OfflineUserDataJobService",
                            "universeDomain": getattr(
                                self._transport._credentials,
                                "universe_domain",
                                "",
                            ),
                            "credentialsType": f"{type(self._transport._credentials).__module__}.{type(self._transport._credentials).__qualname__}",
                            "credentialsInfo": getattr(
                                self.transport._credentials,
                                "get_cred_info",
                                lambda: None,
                            )(),
                        }
                        if hasattr(self._transport, "_credentials")
                        else {
                            "serviceName": "google.ads.googleads.v19.services.OfflineUserDataJobService",
                            "credentialsType": None,
                        }
                    ),
                )

    def create_offline_user_data_job(
        self,
        request: Optional[
            Union[
                offline_user_data_job_service.CreateOfflineUserDataJobRequest,
                dict,
            ]
        ] = None,
        *,
        customer_id: Optional[str] = None,
        job: Optional[offline_user_data_job.OfflineUserDataJob] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> offline_user_data_job_service.CreateOfflineUserDataJobResponse:
        r"""Creates an offline user data job.

        List of thrown errors: `AuthenticationError <>`__
        `AuthorizationError <>`__ `DatabaseError <>`__ `FieldError <>`__
        `HeaderError <>`__ `InternalError <>`__
        `NotAllowlistedError <>`__ `OfflineUserDataJobError <>`__
        `QuotaError <>`__ `RequestError <>`__

        Args:
            request (Union[google.ads.googleads.v19.services.types.CreateOfflineUserDataJobRequest, dict]):
                The request object. Request message for
                [OfflineUserDataJobService.CreateOfflineUserDataJob][google.ads.googleads.v19.services.OfflineUserDataJobService.CreateOfflineUserDataJob].
            customer_id (str):
                Required. The ID of the customer for
                which to create an offline user data
                job.

                This corresponds to the ``customer_id`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            job (google.ads.googleads.v19.resources.types.OfflineUserDataJob):
                Required. The offline user data job
                to be created.

                This corresponds to the ``job`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            google.ads.googleads.v19.services.types.CreateOfflineUserDataJobResponse:
                Response message for
                   [OfflineUserDataJobService.CreateOfflineUserDataJob][google.ads.googleads.v19.services.OfflineUserDataJobService.CreateOfflineUserDataJob].

        """
        # Create or coerce a protobuf request object.
        # - Quick check: If we got a request object, we should *not* have
        #   gotten any keyword arguments that map to the request.
        flattened_params = [customer_id, job]
        has_flattened_params = (
            len([param for param in flattened_params if param is not None]) > 0
        )
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(
            request,
            offline_user_data_job_service.CreateOfflineUserDataJobRequest,
        ):
            request = (
                offline_user_data_job_service.CreateOfflineUserDataJobRequest(
                    request
                )
            )
            # If we have keyword arguments corresponding to fields on the
            # request, apply these.
            if customer_id is not None:
                request.customer_id = customer_id
            if job is not None:
                request.job = job

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._transport._wrapped_methods[
            self._transport.create_offline_user_data_job
        ]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata(
                (("customer_id", request.customer_id),)
            ),
        )

        # Validate the universe domain.
        self._validate_universe_domain()

        # Send the request.
        response = rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    def add_offline_user_data_job_operations(
        self,
        request: Optional[
            Union[
                offline_user_data_job_service.AddOfflineUserDataJobOperationsRequest,
                dict,
            ]
        ] = None,
        *,
        resource_name: Optional[str] = None,
        operations: Optional[
            MutableSequence[
                offline_user_data_job_service.OfflineUserDataJobOperation
            ]
        ] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> offline_user_data_job_service.AddOfflineUserDataJobOperationsResponse:
        r"""Adds operations to the offline user data job.

        List of thrown errors: `AuthenticationError <>`__
        `AuthorizationError <>`__ `DatabaseError <>`__ `FieldError <>`__
        `HeaderError <>`__ `InternalError <>`__ `MutateError <>`__
        `OfflineUserDataJobError <>`__ `QuotaError <>`__
        `RequestError <>`__

        Args:
            request (Union[google.ads.googleads.v19.services.types.AddOfflineUserDataJobOperationsRequest, dict]):
                The request object. Request message for
                [OfflineUserDataJobService.AddOfflineUserDataJobOperations][google.ads.googleads.v19.services.OfflineUserDataJobService.AddOfflineUserDataJobOperations].
            resource_name (str):
                Required. The resource name of the
                OfflineUserDataJob.

                This corresponds to the ``resource_name`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            operations (MutableSequence[google.ads.googleads.v19.services.types.OfflineUserDataJobOperation]):
                Required. The list of operations to
                be done.

                This corresponds to the ``operations`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            google.ads.googleads.v19.services.types.AddOfflineUserDataJobOperationsResponse:
                Response message for
                   [OfflineUserDataJobService.AddOfflineUserDataJobOperations][google.ads.googleads.v19.services.OfflineUserDataJobService.AddOfflineUserDataJobOperations].

        """
        # Create or coerce a protobuf request object.
        # - Quick check: If we got a request object, we should *not* have
        #   gotten any keyword arguments that map to the request.
        flattened_params = [resource_name, operations]
        has_flattened_params = (
            len([param for param in flattened_params if param is not None]) > 0
        )
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(
            request,
            offline_user_data_job_service.AddOfflineUserDataJobOperationsRequest,
        ):
            request = offline_user_data_job_service.AddOfflineUserDataJobOperationsRequest(
                request
            )
            # If we have keyword arguments corresponding to fields on the
            # request, apply these.
            if resource_name is not None:
                request.resource_name = resource_name
            if operations is not None:
                request.operations = operations

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._transport._wrapped_methods[
            self._transport.add_offline_user_data_job_operations
        ]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata(
                (("resource_name", request.resource_name),)
            ),
        )

        # Validate the universe domain.
        self._validate_universe_domain()

        # Send the request.
        response = rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    def run_offline_user_data_job(
        self,
        request: Optional[
            Union[
                offline_user_data_job_service.RunOfflineUserDataJobRequest, dict
            ]
        ] = None,
        *,
        resource_name: Optional[str] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> operation.Operation:
        r"""Runs the offline user data job.

        When finished, the long running operation will contain the
        processing result or failure information, if any.

        List of thrown errors: `AuthenticationError <>`__
        `AuthorizationError <>`__ `DatabaseError <>`__
        `HeaderError <>`__ `InternalError <>`__
        `OfflineUserDataJobError <>`__ `QuotaError <>`__
        `RequestError <>`__

        Args:
            request (Union[google.ads.googleads.v19.services.types.RunOfflineUserDataJobRequest, dict]):
                The request object. Request message for
                [OfflineUserDataJobService.RunOfflineUserDataJob][google.ads.googleads.v19.services.OfflineUserDataJobService.RunOfflineUserDataJob].
            resource_name (str):
                Required. The resource name of the
                OfflineUserDataJob to run.

                This corresponds to the ``resource_name`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            google.api_core.operation.Operation:
                An object representing a long-running operation.

                The result type for the operation will be :class:`google.protobuf.empty_pb2.Empty` A generic empty message that you can re-use to avoid defining duplicated
                   empty messages in your APIs. A typical example is to
                   use it as the request or the response type of an API
                   method. For instance:

                      service Foo {
                         rpc Bar(google.protobuf.Empty) returns
                         (google.protobuf.Empty);

                      }

        """
        # Create or coerce a protobuf request object.
        # - Quick check: If we got a request object, we should *not* have
        #   gotten any keyword arguments that map to the request.
        flattened_params = [resource_name]
        has_flattened_params = (
            len([param for param in flattened_params if param is not None]) > 0
        )
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(
            request, offline_user_data_job_service.RunOfflineUserDataJobRequest
        ):
            request = (
                offline_user_data_job_service.RunOfflineUserDataJobRequest(
                    request
                )
            )
            # If we have keyword arguments corresponding to fields on the
            # request, apply these.
            if resource_name is not None:
                request.resource_name = resource_name

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._transport._wrapped_methods[
            self._transport.run_offline_user_data_job
        ]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata(
                (("resource_name", request.resource_name),)
            ),
        )

        # Validate the universe domain.
        self._validate_universe_domain()

        # Send the request.
        response = rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Wrap the response in an operation future.
        response = operation.from_gapic(
            response,
            self._transport.operations_client,
            empty_pb2.Empty,
            metadata_type=offline_user_data_job.OfflineUserDataJobMetadata,
        )

        # Done; return the response.
        return response

    def __enter__(self) -> "OfflineUserDataJobServiceClient":
        return self

    def __exit__(self, type, value, traceback):
        """Releases underlying transport's resources.

        .. warning::
            ONLY use as a context manager if the transport is NOT shared
            with other clients! Exiting the with block will CLOSE the transport
            and may cause errors in other clients!
        """
        self.transport.close()


DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(
    gapic_version=package_version.__version__
)

if hasattr(DEFAULT_CLIENT_INFO, "protobuf_runtime_version"):  # pragma: NO COVER
    DEFAULT_CLIENT_INFO.protobuf_runtime_version = google.protobuf.__version__

__all__ = ("OfflineUserDataJobServiceClient",)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.errors",
    marshal="google.ads.googleads.v19",
    manifest={
        "CustomConversionGoalErrorEnum",
    },
)


class CustomConversionGoalErrorEnum(proto.Message):
    r"""Container for enum describing possible custom conversion goal
    errors.

    """

    class CustomConversionGoalError(proto.Enum):
        r"""Enum describing possible custom conversion goal errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            INVALID_CONVERSION_ACTION (2):
                Cannot find a conversion action with the
                specified id.
            CONVERSION_ACTION_NOT_ENABLED (3):
                The conversion action is not enabled so it
                cannot be included in a custom conversion goal.
            CANNOT_REMOVE_LINKED_CUSTOM_CONVERSION_GOAL (4):
                The custom conversion goal cannot be removed
                because it's linked to a campaign.
            CUSTOM_GOAL_DUPLICATE_NAME (5):
                Custom goal with the same name already
                exists.
            DUPLICATE_CONVERSION_ACTION_LIST (6):
                Custom goal with the same conversion action
                list already exists.
            NON_BIDDABLE_CONVERSION_ACTION_NOT_ELIGIBLE_FOR_CUSTOM_GOAL (7):
                Conversion types that cannot be biddable
                should not be included in custom goal.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        INVALID_CONVERSION_ACTION = 2
        CONVERSION_ACTION_NOT_ENABLED = 3
        CANNOT_REMOVE_LINKED_CUSTOM_CONVERSION_GOAL = 4
        CUSTOM_GOAL_DUPLICATE_NAME = 5
        DUPLICATE_CONVERSION_ACTION_LIST = 6
        NON_BIDDABLE_CONVERSION_ACTION_NOT_ELIGIBLE_FOR_CUSTOM_GOAL = 7


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v19.enums.types import account_link_status
from google.ads.googleads.v19.enums.types import linked_account_type
from google.ads.googleads.v19.enums.types import mobile_app_vendor


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.resources",
    marshal="google.ads.googleads.v19",
    manifest={
        "AccountLink",
        "ThirdPartyAppAnalyticsLinkIdentifier",
    },
)


class AccountLink(proto.Message):
    r"""Represents the data sharing connection between a Google Ads
    account and another account


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Immutable. Resource name of the account link. AccountLink
            resource names have the form:
            ``customers/{customer_id}/accountLinks/{account_link_id}``
        account_link_id (int):
            Output only. The ID of the link.
            This field is read only.

            This field is a member of `oneof`_ ``_account_link_id``.
        status (google.ads.googleads.v19.enums.types.AccountLinkStatusEnum.AccountLinkStatus):
            The status of the link.
        type_ (google.ads.googleads.v19.enums.types.LinkedAccountTypeEnum.LinkedAccountType):
            Output only. The type of the linked account.
        third_party_app_analytics (google.ads.googleads.v19.resources.types.ThirdPartyAppAnalyticsLinkIdentifier):
            Immutable. A third party app analytics link.

            This field is a member of `oneof`_ ``linked_account``.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    account_link_id: int = proto.Field(
        proto.INT64,
        number=8,
        optional=True,
    )
    status: account_link_status.AccountLinkStatusEnum.AccountLinkStatus = (
        proto.Field(
            proto.ENUM,
            number=3,
            enum=account_link_status.AccountLinkStatusEnum.AccountLinkStatus,
        )
    )
    type_: linked_account_type.LinkedAccountTypeEnum.LinkedAccountType = (
        proto.Field(
            proto.ENUM,
            number=4,
            enum=linked_account_type.LinkedAccountTypeEnum.LinkedAccountType,
        )
    )
    third_party_app_analytics: "ThirdPartyAppAnalyticsLinkIdentifier" = (
        proto.Field(
            proto.MESSAGE,
            number=5,
            oneof="linked_account",
            message="ThirdPartyAppAnalyticsLinkIdentifier",
        )
    )


class ThirdPartyAppAnalyticsLinkIdentifier(proto.Message):
    r"""The identifiers of a Third Party App Analytics Link.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        app_analytics_provider_id (int):
            Immutable. The ID of the app analytics
            provider. This field should not be empty when
            creating a new third party app analytics link.
            It is unable to be modified after the creation
            of the link.

            This field is a member of `oneof`_ ``_app_analytics_provider_id``.
        app_id (str):
            Immutable. A string that uniquely identifies
            a mobile application from which the data was
            collected to the Google Ads API. For iOS, the ID
            string is the 9 digit string that appears at the
            end of an App Store URL (for example,
            "422689480" for "Gmail" whose App Store link is
            https://apps.apple.com/us/app/gmail-email-by-google/id422689480).
            For Android, the ID string is the application's
            package name (for example,
            "com.google.android.gm" for "Gmail" given Google
            Play link
            https://play.google.com/store/apps/details?id=com.google.android.gm)
            This field should not be empty when creating a
            new third party app analytics link. It is unable
            to be modified after the creation of the link.

            This field is a member of `oneof`_ ``_app_id``.
        app_vendor (google.ads.googleads.v19.enums.types.MobileAppVendorEnum.MobileAppVendor):
            Immutable. The vendor of the app.
            This field should not be empty when creating a
            new third party app analytics link. It is unable
            to be modified after the creation of the link.
    """

    app_analytics_provider_id: int = proto.Field(
        proto.INT64,
        number=4,
        optional=True,
    )
    app_id: str = proto.Field(
        proto.STRING,
        number=5,
        optional=True,
    )
    app_vendor: mobile_app_vendor.MobileAppVendorEnum.MobileAppVendor = (
        proto.Field(
            proto.ENUM,
            number=3,
            enum=mobile_app_vendor.MobileAppVendorEnum.MobileAppVendor,
        )
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.enums",
    marshal="google.ads.googleads.v20",
    manifest={
        "CampaignPrimaryStatusEnum",
    },
)


class CampaignPrimaryStatusEnum(proto.Message):
    r"""Container for enum describing possible campaign primary
    status.

    """

    class CampaignPrimaryStatus(proto.Enum):
        r"""Enum describing the possible campaign primary status.
        Provides insight into why a campaign is not serving or not
        serving optimally. Modification to the campaign and its related
        entities might take a while to be reflected in this status.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            ELIGIBLE (2):
                The campaign is eligible to serve.
            PAUSED (3):
                The user-specified campaign status is paused.
            REMOVED (4):
                The user-specified campaign status is
                removed.
            ENDED (5):
                The user-specified time for this campaign to
                end has passed.
            PENDING (6):
                The campaign may serve in the future.
            MISCONFIGURED (7):
                The campaign or its associated entities have
                incorrect user-specified settings.
            LIMITED (8):
                The campaign or its associated entities are
                limited by user-specified settings.
            LEARNING (9):
                The automated bidding system is adjusting to
                user-specified changes to the campaign or
                associated entities.
            NOT_ELIGIBLE (10):
                The campaign is not eligible to serve.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        ELIGIBLE = 2
        PAUSED = 3
        REMOVED = 4
        ENDED = 5
        PENDING = 6
        MISCONFIGURED = 7
        LIMITED = 8
        LEARNING = 9
        NOT_ELIGIBLE = 10


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v22.resources.types import account_budget_proposal
from google.protobuf import field_mask_pb2  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.services",
    marshal="google.ads.googleads.v22",
    manifest={
        "MutateAccountBudgetProposalRequest",
        "AccountBudgetProposalOperation",
        "MutateAccountBudgetProposalResponse",
        "MutateAccountBudgetProposalResult",
    },
)


class MutateAccountBudgetProposalRequest(proto.Message):
    r"""Request message for
    [AccountBudgetProposalService.MutateAccountBudgetProposal][google.ads.googleads.v22.services.AccountBudgetProposalService.MutateAccountBudgetProposal].

    Attributes:
        customer_id (str):
            Required. The ID of the customer.
        operation (google.ads.googleads.v22.services.types.AccountBudgetProposalOperation):
            Required. The operation to perform on an
            individual account-level budget proposal.
        validate_only (bool):
            If true, the request is validated but not
            executed. Only errors are returned, not results.
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    operation: "AccountBudgetProposalOperation" = proto.Field(
        proto.MESSAGE,
        number=2,
        message="AccountBudgetProposalOperation",
    )
    validate_only: bool = proto.Field(
        proto.BOOL,
        number=3,
    )


class AccountBudgetProposalOperation(proto.Message):
    r"""A single operation to propose the creation of a new
    account-level budget or edit/end/remove an existing one.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        update_mask (google.protobuf.field_mask_pb2.FieldMask):
            FieldMask that determines which budget fields
            are modified.  While budgets may be modified,
            proposals that propose such modifications are
            final. Therefore, update operations are not
            supported for proposals.

            Proposals that modify budgets have the 'update'
            proposal type.  Specifying a mask for any other
            proposal type is considered an error.
        create (google.ads.googleads.v22.resources.types.AccountBudgetProposal):
            Create operation: A new proposal to create a
            new budget, edit an existing budget, end an
            actively running budget, or remove an approved
            budget scheduled to start in the future.
            No resource name is expected for the new
            proposal.

            This field is a member of `oneof`_ ``operation``.
        remove (str):
            Remove operation: A resource name for the removed proposal
            is expected, in this format:

            ``customers/{customer_id}/accountBudgetProposals/{account_budget_proposal_id}``
            A request may be cancelled iff it is pending.

            This field is a member of `oneof`_ ``operation``.
    """

    update_mask: field_mask_pb2.FieldMask = proto.Field(
        proto.MESSAGE,
        number=3,
        message=field_mask_pb2.FieldMask,
    )
    create: account_budget_proposal.AccountBudgetProposal = proto.Field(
        proto.MESSAGE,
        number=2,
        oneof="operation",
        message=account_budget_proposal.AccountBudgetProposal,
    )
    remove: str = proto.Field(
        proto.STRING,
        number=1,
        oneof="operation",
    )


class MutateAccountBudgetProposalResponse(proto.Message):
    r"""Response message for account-level budget mutate operations.

    Attributes:
        result (google.ads.googleads.v22.services.types.MutateAccountBudgetProposalResult):
            The result of the mutate.
    """

    result: "MutateAccountBudgetProposalResult" = proto.Field(
        proto.MESSAGE,
        number=2,
        message="MutateAccountBudgetProposalResult",
    )


class MutateAccountBudgetProposalResult(proto.Message):
    r"""The result for the account budget proposal mutate.

    Attributes:
        resource_name (str):
            Returned for successful operations.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



#!/usr/bin/env python
# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""This example generates keyword ideas from a list of seed keywords."""


import argparse
import sys
from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.v22.enums.types.keyword_plan_competition_level import (
    KeywordPlanCompetitionLevelEnum,
)
from google.ads.googleads.v22.enums.types.keyword_plan_network import (
    KeywordPlanNetworkEnum,
)
from google.ads.googleads.v22.services.services.geo_target_constant_service.client import (
    GeoTargetConstantServiceClient,
)
from google.ads.googleads.v22.services.services.google_ads_service.client import (
    GoogleAdsServiceClient,
)
from google.ads.googleads.v22.services.services.keyword_plan_idea_service.client import (
    KeywordPlanIdeaServiceClient,
)
from google.ads.googleads.v22.services.types.keyword_plan_idea_service import (
    GenerateKeywordIdeasRequest,
    GenerateKeywordIdeaResult,
)

# Location IDs are listed here:
# https://developers.google.com/google-ads/api/reference/data/geotargets
# and they can also be retrieved using the GeoTargetConstantService as shown
# here: https://developers.google.com/google-ads/api/docs/targeting/location-targeting
_DEFAULT_LOCATION_IDS = ["1023191"]  # location ID for New York, NY
# A language criterion ID. For example, specify 1000 for English. For more
# information on determining this value, see the below link:
# https://developers.google.com/google-ads/api/reference/data/codes-formats#expandable-7
_DEFAULT_LANGUAGE_ID = "1000"  # language ID for English


# [START generate_keyword_ideas]
def main(
    client: GoogleAdsClient,
    customer_id: str,
    location_ids: list[str],
    language_id: str,
    keyword_texts: list[str],
    page_url: str,
):
    keyword_plan_idea_service: KeywordPlanIdeaServiceClient = (
        client.get_service("KeywordPlanIdeaService")
    )
    keyword_competition_level_enum: KeywordPlanCompetitionLevelEnum = (
        client.enums.KeywordPlanCompetitionLevelEnum
    )
    keyword_plan_network: KeywordPlanNetworkEnum = (
        client.enums.KeywordPlanNetworkEnum.GOOGLE_SEARCH_AND_PARTNERS
    )
    location_rns: list[str] = map_locations_ids_to_resource_names(
        client, location_ids
    )
    google_ads_service: GoogleAdsServiceClient = client.get_service(
        "GoogleAdsService"
    )
    language_rn: str = google_ads_service.language_constant_path(language_id)

    # Either keywords or a page_url are required to generate keyword ideas
    # so this raises an error if neither are provided.
    if not (keyword_texts or page_url):
        raise ValueError(
            "At least one of keywords or page URL is required, "
            "but neither was specified."
        )

    # Only one of the fields "url_seed", "keyword_seed", or
    # "keyword_and_url_seed" can be set on the request, depending on whether
    # keywords, a page_url or both were passed to this function.
    request: GenerateKeywordIdeasRequest = client.get_type(
        "GenerateKeywordIdeasRequest"
    )
    request.customer_id = customer_id
    request.language = language_rn
    request.geo_target_constants = location_rns
    request.include_adult_keywords = False
    request.keyword_plan_network = keyword_plan_network

    # To generate keyword ideas with only a page_url and no keywords we need
    # to initialize a UrlSeed object with the page_url as the "url" field.
    if not keyword_texts and page_url:
        request.url_seed.url = page_url

    # To generate keyword ideas with only a list of keywords and no page_url
    # we need to initialize a KeywordSeed object and set the "keywords" field
    # to be a list of StringValue objects.
    if keyword_texts and not page_url:
        request.keyword_seed.keywords.extend(keyword_texts)

    # To generate keyword ideas using both a list of keywords and a page_url we
    # need to initialize a KeywordAndUrlSeed object, setting both the "url" and
    # "keywords" fields.
    if keyword_texts and page_url:
        request.keyword_and_url_seed.url = page_url
        request.keyword_and_url_seed.keywords.extend(keyword_texts)

    keyword_ideas = keyword_plan_idea_service.generate_keyword_ideas(
        request=request
    )

    idea: GenerateKeywordIdeaResult
    for idea in keyword_ideas:
        competition_value = idea.keyword_idea_metrics.competition.name
        print(
            f'Keyword idea text "{idea.text}" has '
            f'"{idea.keyword_idea_metrics.avg_monthly_searches}" '
            f'average monthly searches and "{competition_value}" '
            "competition.\n"
        )
    # [END generate_keyword_ideas]


def map_locations_ids_to_resource_names(
    client: GoogleAdsClient, location_ids: list[str]
) -> list[str]:
    """Converts a list of location IDs to resource names.

    Args:
        client: an initialized GoogleAdsClient instance.
        location_ids: a list of location ID strings.

    Returns:
        a list of resource name strings using the given location IDs.
    """
    geo_target_constant_service: GeoTargetConstantServiceClient = (
        client.get_service("GeoTargetConstantService")
    )
    build_resource_name = geo_target_constant_service.geo_target_constant_path
    return [build_resource_name(location_id) for location_id in location_ids]


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description="Generates keyword ideas from a list of seed keywords."
    )

    # The following argument(s) should be provided to run the example.
    parser.add_argument(
        "-c",
        "--customer_id",
        type=str,
        required=True,
        help="The Google Ads customer ID.",
    )
    parser.add_argument(
        "-k",
        "--keyword_texts",
        nargs="+",
        type=str,
        required=False,
        default=[],
        help="Space-delimited list of starter keywords",
    )
    # To determine the appropriate location IDs, see:
    # https://developers.google.com/google-ads/api/reference/data/geotargets
    parser.add_argument(
        "-l",
        "--location_ids",
        nargs="+",
        type=str,
        required=False,
        default=_DEFAULT_LOCATION_IDS,
        help="Space-delimited list of location criteria IDs",
    )
    # To determine the appropriate language ID, see:
    # https://developers.google.com/google-ads/api/data/codes-formats#languages
    parser.add_argument(
        "-i",
        "--language_id",
        type=str,
        required=False,
        default=_DEFAULT_LANGUAGE_ID,
        help="The language criterion ID.",
    )
    # Optional: Specify a URL string related to your business to generate ideas.
    parser.add_argument(
        "-p",
        "--page_url",
        type=str,
        required=False,
        help="A URL string related to your business",
    )

    args = parser.parse_args()

    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client = GoogleAdsClient.load_from_storage(version="v22")

    try:
        main(
            googleads_client,
            args.customer_id,
            args.location_ids,
            args.language_id,
            args.keyword_texts,
            args.page_url,
        )
    except GoogleAdsException as ex:
        print(
            f'Request with ID "{ex.request_id}" failed with status '
            f'"{ex.error.code().name}" and includes the following errors:'
        )
        for error in ex.failure.errors:
            print(f'\tError with message "{error.message}".')
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(f"\t\tOn field: {field_path_element.field_name}")
        sys.exit(1)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v21.enums.types import lead_form_field_user_input_type


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.resources",
    marshal="google.ads.googleads.v21",
    manifest={
        "LeadFormSubmissionData",
        "LeadFormSubmissionField",
        "CustomLeadFormSubmissionField",
    },
)


class LeadFormSubmissionData(proto.Message):
    r"""Data from lead form submissions.

    Attributes:
        resource_name (str):
            Output only. The resource name of the lead form submission
            data. Lead form submission data resource names have the
            form:

            ``customers/{customer_id}/leadFormSubmissionData/{lead_form_submission_data_id}``
        id (str):
            Output only. ID of this lead form submission.
        asset (str):
            Output only. Asset associated with the
            submitted lead form.
        campaign (str):
            Output only. Campaign associated with the
            submitted lead form.
        lead_form_submission_fields (MutableSequence[google.ads.googleads.v21.resources.types.LeadFormSubmissionField]):
            Output only. Submission data associated with
            a lead form.
        custom_lead_form_submission_fields (MutableSequence[google.ads.googleads.v21.resources.types.CustomLeadFormSubmissionField]):
            Output only. Submission data associated with
            a custom lead form.
        ad_group (str):
            Output only. AdGroup associated with the
            submitted lead form.
        ad_group_ad (str):
            Output only. AdGroupAd associated with the
            submitted lead form.
        gclid (str):
            Output only. Google Click Id associated with
            the submissed lead form.
        submission_date_time (str):
            Output only. The date and time at which the lead form was
            submitted. The format is "yyyy-mm-dd hh:mm:ss+|-hh:mm", for
            example, "2019-01-01 12:32:45-08:00".
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    id: str = proto.Field(
        proto.STRING,
        number=2,
    )
    asset: str = proto.Field(
        proto.STRING,
        number=3,
    )
    campaign: str = proto.Field(
        proto.STRING,
        number=4,
    )
    lead_form_submission_fields: MutableSequence["LeadFormSubmissionField"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=5,
            message="LeadFormSubmissionField",
        )
    )
    custom_lead_form_submission_fields: MutableSequence[
        "CustomLeadFormSubmissionField"
    ] = proto.RepeatedField(
        proto.MESSAGE,
        number=10,
        message="CustomLeadFormSubmissionField",
    )
    ad_group: str = proto.Field(
        proto.STRING,
        number=6,
    )
    ad_group_ad: str = proto.Field(
        proto.STRING,
        number=7,
    )
    gclid: str = proto.Field(
        proto.STRING,
        number=8,
    )
    submission_date_time: str = proto.Field(
        proto.STRING,
        number=9,
    )


class LeadFormSubmissionField(proto.Message):
    r"""Fields in the submitted lead form.

    Attributes:
        field_type (google.ads.googleads.v21.enums.types.LeadFormFieldUserInputTypeEnum.LeadFormFieldUserInputType):
            Output only. Field type for lead form fields.
        field_value (str):
            Output only. Field value for lead form
            fields.
    """

    field_type: (
        lead_form_field_user_input_type.LeadFormFieldUserInputTypeEnum.LeadFormFieldUserInputType
    ) = proto.Field(
        proto.ENUM,
        number=1,
        enum=lead_form_field_user_input_type.LeadFormFieldUserInputTypeEnum.LeadFormFieldUserInputType,
    )
    field_value: str = proto.Field(
        proto.STRING,
        number=2,
    )


class CustomLeadFormSubmissionField(proto.Message):
    r"""Fields in the submitted custom question

    Attributes:
        question_text (str):
            Output only. Question text for custom
            question, maximum number of characters is 300.
        field_value (str):
            Output only. Field value for custom question
            response, maximum number of characters is 70.
    """

    question_text: str = proto.Field(
        proto.STRING,
        number=1,
    )
    field_value: str = proto.Field(
        proto.STRING,
        number=2,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.enums",
    marshal="google.ads.googleads.v19",
    manifest={
        "KeywordPlanForecastIntervalEnum",
    },
)


class KeywordPlanForecastIntervalEnum(proto.Message):
    r"""Container for enumeration of forecast intervals."""

    class KeywordPlanForecastInterval(proto.Enum):
        r"""Forecast intervals.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                The value is unknown in this version.
            NEXT_WEEK (3):
                The next week date range for keyword plan.
                The next week is based on the default locale of
                the user's account and is mostly SUN-SAT or
                MON-SUN.
                This can be different from next-7 days.
            NEXT_MONTH (4):
                The next month date range for keyword plan.
            NEXT_QUARTER (5):
                The next quarter date range for keyword plan.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        NEXT_WEEK = 3
        NEXT_MONTH = 4
        NEXT_QUARTER = 5


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v19.common.types import criteria
from google.ads.googleads.v19.enums.types import audience_insights_dimension
from google.ads.googleads.v19.enums.types import (
    insights_knowledge_graph_entity_capabilities,
)


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.common",
    marshal="google.ads.googleads.v19",
    manifest={
        "AudienceInsightsAttributeMetadata",
        "AudienceInsightsAttribute",
        "AudienceInsightsTopic",
        "AudienceInsightsEntity",
        "AudienceInsightsCategory",
        "AudienceInsightsDynamicLineup",
        "YouTubeChannelAttributeMetadata",
        "YouTubeVideoAttributeMetadata",
        "DynamicLineupAttributeMetadata",
        "LocationAttributeMetadata",
        "UserInterestAttributeMetadata",
        "KnowledgeGraphAttributeMetadata",
    },
)


class AudienceInsightsAttributeMetadata(proto.Message):
    r"""An audience attribute, with metadata about it, returned in
    response to a search.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        dimension (google.ads.googleads.v19.enums.types.AudienceInsightsDimensionEnum.AudienceInsightsDimension):
            The type of the attribute.
        attribute (google.ads.googleads.v19.common.types.AudienceInsightsAttribute):
            The attribute itself.
        display_name (str):
            The human-readable name of the attribute.
        display_info (str):
            A string that supplements the display_name to identify the
            attribute. If the dimension is TOPIC, this is a brief
            description of the Knowledge Graph entity, such as "American
            singer-songwriter". If the dimension is CATEGORY, this is
            the complete path to the category in The Product & Service
            taxonomy, for example "/Apparel/Clothing/Outerwear".
        potential_youtube_reach (int):
            An estimate of the number of reachable
            YouTube users matching this attribute in the
            requested location, or zero if that information
            is not available for this attribute. This field
            is not populated in every response.
        subscriber_share (float):
            The share of subscribers within this
            attribute, between and including 0 and
            1. This field is not populated in every
                response.
        viewer_share (float):
            The share of viewers within this attribute,
            between and including 0 and
            1. This field is not populated in every
                response.
        youtube_channel_metadata (google.ads.googleads.v19.common.types.YouTubeChannelAttributeMetadata):
            Special metadata for a YouTube channel.

            This field is a member of `oneof`_ ``dimension_metadata``.
        youtube_video_metadata (google.ads.googleads.v19.common.types.YouTubeVideoAttributeMetadata):
            Special metadata for a YouTube video.

            This field is a member of `oneof`_ ``dimension_metadata``.
        dynamic_attribute_metadata (google.ads.googleads.v19.common.types.DynamicLineupAttributeMetadata):
            Special metadata for a YouTube Dynamic
            Lineup.

            This field is a member of `oneof`_ ``dimension_metadata``.
        location_attribute_metadata (google.ads.googleads.v19.common.types.LocationAttributeMetadata):
            Special metadata for a Location.

            This field is a member of `oneof`_ ``dimension_metadata``.
        user_interest_attribute_metadata (google.ads.googleads.v19.common.types.UserInterestAttributeMetadata):
            Special metadata for a User Interest.

            This field is a member of `oneof`_ ``dimension_metadata``.
        knowledge_graph_attribute_metadata (google.ads.googleads.v19.common.types.KnowledgeGraphAttributeMetadata):
            Special metadata for a Knowledge Graph
            Entity.

            This field is a member of `oneof`_ ``dimension_metadata``.
    """

    dimension: (
        audience_insights_dimension.AudienceInsightsDimensionEnum.AudienceInsightsDimension
    ) = proto.Field(
        proto.ENUM,
        number=1,
        enum=audience_insights_dimension.AudienceInsightsDimensionEnum.AudienceInsightsDimension,
    )
    attribute: "AudienceInsightsAttribute" = proto.Field(
        proto.MESSAGE,
        number=2,
        message="AudienceInsightsAttribute",
    )
    display_name: str = proto.Field(
        proto.STRING,
        number=3,
    )
    display_info: str = proto.Field(
        proto.STRING,
        number=4,
    )
    potential_youtube_reach: int = proto.Field(
        proto.INT64,
        number=8,
    )
    subscriber_share: float = proto.Field(
        proto.DOUBLE,
        number=9,
    )
    viewer_share: float = proto.Field(
        proto.DOUBLE,
        number=13,
    )
    youtube_channel_metadata: "YouTubeChannelAttributeMetadata" = proto.Field(
        proto.MESSAGE,
        number=5,
        oneof="dimension_metadata",
        message="YouTubeChannelAttributeMetadata",
    )
    youtube_video_metadata: "YouTubeVideoAttributeMetadata" = proto.Field(
        proto.MESSAGE,
        number=10,
        oneof="dimension_metadata",
        message="YouTubeVideoAttributeMetadata",
    )
    dynamic_attribute_metadata: "DynamicLineupAttributeMetadata" = proto.Field(
        proto.MESSAGE,
        number=6,
        oneof="dimension_metadata",
        message="DynamicLineupAttributeMetadata",
    )
    location_attribute_metadata: "LocationAttributeMetadata" = proto.Field(
        proto.MESSAGE,
        number=7,
        oneof="dimension_metadata",
        message="LocationAttributeMetadata",
    )
    user_interest_attribute_metadata: "UserInterestAttributeMetadata" = (
        proto.Field(
            proto.MESSAGE,
            number=11,
            oneof="dimension_metadata",
            message="UserInterestAttributeMetadata",
        )
    )
    knowledge_graph_attribute_metadata: "KnowledgeGraphAttributeMetadata" = (
        proto.Field(
            proto.MESSAGE,
            number=12,
            oneof="dimension_metadata",
            message="KnowledgeGraphAttributeMetadata",
        )
    )


class AudienceInsightsAttribute(proto.Message):
    r"""An audience attribute that can be used to request insights
    about the audience.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        age_range (google.ads.googleads.v19.common.types.AgeRangeInfo):
            An audience attribute defined by an age
            range.

            This field is a member of `oneof`_ ``attribute``.
        gender (google.ads.googleads.v19.common.types.GenderInfo):
            An audience attribute defined by a gender.

            This field is a member of `oneof`_ ``attribute``.
        location (google.ads.googleads.v19.common.types.LocationInfo):
            An audience attribute defined by a geographic
            location.

            This field is a member of `oneof`_ ``attribute``.
        user_interest (google.ads.googleads.v19.common.types.UserInterestInfo):
            An Affinity or In-Market audience.

            This field is a member of `oneof`_ ``attribute``.
        entity (google.ads.googleads.v19.common.types.AudienceInsightsEntity):
            An audience attribute defined by interest in
            a topic represented by a Knowledge Graph entity.

            This field is a member of `oneof`_ ``attribute``.
        category (google.ads.googleads.v19.common.types.AudienceInsightsCategory):
            An audience attribute defined by interest in
            a Product & Service category.

            This field is a member of `oneof`_ ``attribute``.
        dynamic_lineup (google.ads.googleads.v19.common.types.AudienceInsightsDynamicLineup):
            A YouTube Dynamic Lineup.

            This field is a member of `oneof`_ ``attribute``.
        parental_status (google.ads.googleads.v19.common.types.ParentalStatusInfo):
            A Parental Status value (parent, or not a
            parent).

            This field is a member of `oneof`_ ``attribute``.
        income_range (google.ads.googleads.v19.common.types.IncomeRangeInfo):
            A household income percentile range.

            This field is a member of `oneof`_ ``attribute``.
        youtube_channel (google.ads.googleads.v19.common.types.YouTubeChannelInfo):
            A YouTube channel.

            This field is a member of `oneof`_ ``attribute``.
        youtube_video (google.ads.googleads.v19.common.types.YouTubeVideoInfo):
            A YouTube video.

            This field is a member of `oneof`_ ``attribute``.
    """

    age_range: criteria.AgeRangeInfo = proto.Field(
        proto.MESSAGE,
        number=1,
        oneof="attribute",
        message=criteria.AgeRangeInfo,
    )
    gender: criteria.GenderInfo = proto.Field(
        proto.MESSAGE,
        number=2,
        oneof="attribute",
        message=criteria.GenderInfo,
    )
    location: criteria.LocationInfo = proto.Field(
        proto.MESSAGE,
        number=3,
        oneof="attribute",
        message=criteria.LocationInfo,
    )
    user_interest: criteria.UserInterestInfo = proto.Field(
        proto.MESSAGE,
        number=4,
        oneof="attribute",
        message=criteria.UserInterestInfo,
    )
    entity: "AudienceInsightsEntity" = proto.Field(
        proto.MESSAGE,
        number=5,
        oneof="attribute",
        message="AudienceInsightsEntity",
    )
    category: "AudienceInsightsCategory" = proto.Field(
        proto.MESSAGE,
        number=6,
        oneof="attribute",
        message="AudienceInsightsCategory",
    )
    dynamic_lineup: "AudienceInsightsDynamicLineup" = proto.Field(
        proto.MESSAGE,
        number=7,
        oneof="attribute",
        message="AudienceInsightsDynamicLineup",
    )
    parental_status: criteria.ParentalStatusInfo = proto.Field(
        proto.MESSAGE,
        number=8,
        oneof="attribute",
        message=criteria.ParentalStatusInfo,
    )
    income_range: criteria.IncomeRangeInfo = proto.Field(
        proto.MESSAGE,
        number=9,
        oneof="attribute",
        message=criteria.IncomeRangeInfo,
    )
    youtube_channel: criteria.YouTubeChannelInfo = proto.Field(
        proto.MESSAGE,
        number=10,
        oneof="attribute",
        message=criteria.YouTubeChannelInfo,
    )
    youtube_video: criteria.YouTubeVideoInfo = proto.Field(
        proto.MESSAGE,
        number=11,
        oneof="attribute",
        message=criteria.YouTubeVideoInfo,
    )


class AudienceInsightsTopic(proto.Message):
    r"""An entity or category representing a topic that defines an
    audience.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        entity (google.ads.googleads.v19.common.types.AudienceInsightsEntity):
            A Knowledge Graph entity

            This field is a member of `oneof`_ ``topic``.
        category (google.ads.googleads.v19.common.types.AudienceInsightsCategory):
            A Product & Service category

            This field is a member of `oneof`_ ``topic``.
    """

    entity: "AudienceInsightsEntity" = proto.Field(
        proto.MESSAGE,
        number=1,
        oneof="topic",
        message="AudienceInsightsEntity",
    )
    category: "AudienceInsightsCategory" = proto.Field(
        proto.MESSAGE,
        number=2,
        oneof="topic",
        message="AudienceInsightsCategory",
    )


class AudienceInsightsEntity(proto.Message):
    r"""A Knowledge Graph entity, represented by its machine id.

    Attributes:
        knowledge_graph_machine_id (str):
            Required. The machine ID (mid) of the
            Knowledge Graph entity.
    """

    knowledge_graph_machine_id: str = proto.Field(
        proto.STRING,
        number=1,
    )


class AudienceInsightsCategory(proto.Message):
    r"""A Product and Service category.

    Attributes:
        category_id (str):
            Required. The criterion ID of the category.
    """

    category_id: str = proto.Field(
        proto.STRING,
        number=1,
    )


class AudienceInsightsDynamicLineup(proto.Message):
    r"""A YouTube Dynamic Lineup.

    Attributes:
        dynamic_lineup_id (str):
            Required. The numeric ID of the dynamic
            lineup.
    """

    dynamic_lineup_id: str = proto.Field(
        proto.STRING,
        number=1,
    )


class YouTubeChannelAttributeMetadata(proto.Message):
    r"""Metadata associated with a YouTube channel attribute.

    Attributes:
        subscriber_count (int):
            The approximate number of subscribers to the
            YouTube channel.
    """

    subscriber_count: int = proto.Field(
        proto.INT64,
        number=1,
    )


class YouTubeVideoAttributeMetadata(proto.Message):
    r"""Metadata for a YouTube video attribute.

    Attributes:
        thumbnail_url (str):
            The URL of the video thumbnail, prefixed by
            "https://img.youtube.com/".
        video_url (str):
            The URL of the video, prefixed by
            "https://www.youtube.com/".
    """

    thumbnail_url: str = proto.Field(
        proto.STRING,
        number=1,
    )
    video_url: str = proto.Field(
        proto.STRING,
        number=2,
    )


class DynamicLineupAttributeMetadata(proto.Message):
    r"""Metadata associated with a Dynamic Lineup attribute.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        inventory_country (google.ads.googleads.v19.common.types.LocationInfo):
            The national market associated with the
            lineup.
        median_monthly_inventory (int):
            The median number of impressions per month on
            this lineup.

            This field is a member of `oneof`_ ``_median_monthly_inventory``.
        channel_count_lower_bound (int):
            The lower end of a range containing the
            number of channels in the lineup.

            This field is a member of `oneof`_ ``_channel_count_lower_bound``.
        channel_count_upper_bound (int):
            The upper end of a range containing the
            number of channels in the lineup.

            This field is a member of `oneof`_ ``_channel_count_upper_bound``.
        sample_channels (MutableSequence[google.ads.googleads.v19.common.types.DynamicLineupAttributeMetadata.SampleChannel]):
            Examples of channels that are included in the
            lineup.
    """

    class SampleChannel(proto.Message):
        r"""A YouTube channel returned as an example of the content in a
        lineup.

        Attributes:
            youtube_channel (google.ads.googleads.v19.common.types.YouTubeChannelInfo):
                A YouTube channel.
            display_name (str):
                The name of the sample channel.
            youtube_channel_metadata (google.ads.googleads.v19.common.types.YouTubeChannelAttributeMetadata):
                Metadata for the sample channel.
        """

        youtube_channel: criteria.YouTubeChannelInfo = proto.Field(
            proto.MESSAGE,
            number=1,
            message=criteria.YouTubeChannelInfo,
        )
        display_name: str = proto.Field(
            proto.STRING,
            number=2,
        )
        youtube_channel_metadata: "YouTubeChannelAttributeMetadata" = (
            proto.Field(
                proto.MESSAGE,
                number=3,
                message="YouTubeChannelAttributeMetadata",
            )
        )

    inventory_country: criteria.LocationInfo = proto.Field(
        proto.MESSAGE,
        number=1,
        message=criteria.LocationInfo,
    )
    median_monthly_inventory: int = proto.Field(
        proto.INT64,
        number=2,
        optional=True,
    )
    channel_count_lower_bound: int = proto.Field(
        proto.INT64,
        number=3,
        optional=True,
    )
    channel_count_upper_bound: int = proto.Field(
        proto.INT64,
        number=4,
        optional=True,
    )
    sample_channels: MutableSequence[SampleChannel] = proto.RepeatedField(
        proto.MESSAGE,
        number=5,
        message=SampleChannel,
    )


class LocationAttributeMetadata(proto.Message):
    r"""Metadata associated with a Location attribute.

    Attributes:
        country_location (google.ads.googleads.v19.common.types.LocationInfo):
            The country location that this attribute’s
            sub country location is located in.
    """

    country_location: criteria.LocationInfo = proto.Field(
        proto.MESSAGE,
        number=1,
        message=criteria.LocationInfo,
    )


class UserInterestAttributeMetadata(proto.Message):
    r"""Metadata associated with a User Interest attribute.

    Attributes:
        user_interest_description (str):
            English language text description of the user
            interest category (200 characters max).
    """

    user_interest_description: str = proto.Field(
        proto.STRING,
        number=1,
    )


class KnowledgeGraphAttributeMetadata(proto.Message):
    r"""Metadata associated with a Knowledge Graph Entity attribute.

    Attributes:
        entity_capabilities (MutableSequence[google.ads.googleads.v19.enums.types.InsightsKnowledgeGraphEntityCapabilitiesEnum.InsightsKnowledgeGraphEntityCapabilities]):
            The capabilities of the entity used in
            [ContentCreatorInsightsService][].
    """

    entity_capabilities: MutableSequence[
        insights_knowledge_graph_entity_capabilities.InsightsKnowledgeGraphEntityCapabilitiesEnum.InsightsKnowledgeGraphEntityCapabilities
    ] = proto.RepeatedField(
        proto.ENUM,
        number=1,
        enum=insights_knowledge_graph_entity_capabilities.InsightsKnowledgeGraphEntityCapabilitiesEnum.InsightsKnowledgeGraphEntityCapabilities,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.enums",
    marshal="google.ads.googleads.v21",
    manifest={
        "CriterionCategoryChannelAvailabilityModeEnum",
    },
)


class CriterionCategoryChannelAvailabilityModeEnum(proto.Message):
    r"""Describes channel availability mode for a criterion
    availability - whether the availability is meant to include all
    advertising channels, or a particular channel with all its
    channel subtypes, or a channel with a certain subset of channel
    subtypes.

    """

    class CriterionCategoryChannelAvailabilityMode(proto.Enum):
        r"""Enum containing the possible
        CriterionCategoryChannelAvailabilityMode.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            ALL_CHANNELS (2):
                The category is available to campaigns of all
                channel types and subtypes.
            CHANNEL_TYPE_AND_ALL_SUBTYPES (3):
                The category is available to campaigns of a
                specific channel type, including all subtypes
                under it.
            CHANNEL_TYPE_AND_SUBSET_SUBTYPES (4):
                The category is available to campaigns of a
                specific channel type and subtype(s).
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        ALL_CHANNELS = 2
        CHANNEL_TYPE_AND_ALL_SUBTYPES = 3
        CHANNEL_TYPE_AND_SUBSET_SUBTYPES = 4


__all__ = tuple(sorted(__protobuf__.manifest))
